// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include <syslog.h>
#include "appsettingsdialog.h"
#include "ui_appsettingsdialog.h"
#include "mainWindow.h"

AppSettingsDialog::AppSettingsDialog(QWidget *parent) :
    CommonDialog(parent),
    ui(new Ui::AppSettingsDialog)
{
    ui->setupUi(this);

    // Make this a hidden feature for now
    ui->logLevelFrame->hide();

    ui->appLoggingCheckBox->setChecked( MainWindow::instance()->getLogging());
    ui->startupTooltipCheckBox->setChecked( MainWindow::instance()->getShowStartupTooltips());
    ui->showHelpStepsCheckBox->setChecked( MainWindow::instance()->getShowHelpSteps());

    setWindowFlags(Qt::FramelessWindowHint | Qt::Dialog);
    setWindowMask( "", width()-11, height()-11);
}

AppSettingsDialog::~AppSettingsDialog()
{
    syslog( LOG_INFO, "Deleting the App Settings Dialog");
    delete ui;
    syslog( LOG_INFO, "Deleted the App Settings Dialog");
}

void AppSettingsDialog::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}


////////////////////////////////////////////////////////////
//
// Automatic Qt event hooks
//
////////////////////////////////////////////////////////////

void AppSettingsDialog::on_okButton_clicked()
{
    bool checked;

    // Save the startup settings
    MainWindow::instance()->setShowStartupTooltips( ui->startupTooltipCheckBox->isChecked());

    //  Deal with the help box settings
    MainWindow::instance()->setShowHelpSteps(ui->showHelpStepsCheckBox->isChecked() );

    //  Deal with the Logging settings
    if ( ui->appLoggingCheckBox->isChecked() )
    {
        setlogmask( LOG_UPTO( LOG_INFO ));
        checked = true;
    }
    else
    {
        setlogmask( LOG_UPTO( LOG_ERR ));
        checked = false;
    }

    MainWindow::instance()->setLogging( checked );

    hide();
}

void AppSettingsDialog::on_cancelButton_clicked()
{
    hide();
}
