// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#ifndef AUTOCONFIGUREDIALOG_H
#define AUTOCONFIGUREDIALOG_H

#include <QDialog>
#include <QBitmap>
#include <QSharedPointer>
#include <QNetworkReply>
#include <QFileDialog>

#include <maskfactory.h>
#include <CommonDialog/commonDialog.h>
#include <syslog.h>


#include "farm.h"
#include "appPortalConfigCollector.h"
#include "appPortalConfigParser.h"
#include "common/applicationSettings.h"
#include "common/common.h"
#include "BasicMessageDialog/basicmessagedialog.h"

namespace Ui {
    class AutoConfigureDialog;
}

class AutoConfigureDialog : public CommonDialog {
    Q_OBJECT

public:
    AutoConfigureDialog(Farm* farm, QWidget *parent = 0);
    ~AutoConfigureDialog();

public slots:
    void networkReply(QNetworkReply::NetworkError error, QString errorString, QString urlString);

private slots:
    void on_backButton_clicked();
    void on_connectButton_clicked();
    void on_cancelButton_clicked();

    void accept();
    void cancel();

    void currentProtocolChanged(int);
    void browseFile();

    void manuallyChecked(bool);

protected:
    void changeEvent(QEvent *e);

private:
    Ui::AutoConfigureDialog *ui;

    QString buildUrl( void);
    void saveSettings();
    void loadSettings();
    bool loadFarm();
    bool parseConfigFromFile(QString dir, QString fileName);

    Farm* farm;
    AppPortalConfigCollector* configCollector;
    AppPortalConfigParser* configParser;
    SharedApplicationSettings m_appSettings;

};

#endif // AUTOCONFIGUREDIALOG_H
