// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include "changePasswordDialog.h"
#include <syslog.h>
#include <QMessageBox>

ChangePasswordDialog::ChangePasswordDialog(FarmSettings *settings, QString * username,
        QString * domain, QString * password, QString * newPassword,
        Farm ** farm, QWidget *parent/* = 0*/)
    : CommonDialog(parent)
{

    Ui_ChangePasswordDialogClass::setupUi(this);

    p_farm = farm;
    p_username = username;
    p_password = password;
    p_newPassword = newPassword;
    p_domain = domain;
    // Update interface
    p_settings = settings;

    // Load all the farms that have password change support configured
    for(int i = 0; i < p_settings->getFarmNamesList().count(); ++i)
    {
        if (p_settings->getFarm(i)->isPasswordManagementAllowed() && !p_settings->getFarm(i)->getPasswordManagementServer().isEmpty())
        {
            farmNameComboBox->addItem(p_settings->getFarm(i)->getName());
        }
    }

    // If a farm is already chosen AND it has password change allowed, then set it.  Else, pick the first one
    if (p_settings->getCurrentFarm() != 0 && p_settings->getCurrentFarm()->isPasswordManagementAllowed() && !p_settings->getCurrentFarm()->getPasswordManagementServer().isEmpty())
    {
        farmNameComboBox->setCurrentIndex(farmNameComboBox->findText(p_settings->getCurrentFarm()->getName()));
    }
    else
    {
        *p_farm = p_settings->getFarm(0);
        farmNameComboBox->setCurrentIndex(0);
    }

    refreshSettings(farmNameComboBox->currentText());

    connect(okButton, SIGNAL(clicked()), SLOT(accept()));
    connect(cancelButton, SIGNAL(clicked()), SLOT(reject()));
    connect(farmNameComboBox, SIGNAL(currentIndexChanged(const QString &)), this, SLOT(refreshSettings(const QString &)));


    setWindowFlags(Qt::FramelessWindowHint | Qt::Dialog);
    setWindowMask( "", width()-11, height()-11);

    syslog( LOG_INFO, "Frame is %ix%i", frame->width(), frame->height());
}

ChangePasswordDialog::~ChangePasswordDialog()
{

}

void ChangePasswordDialog::refreshSettings(const QString &text)
{
    *p_farm = p_settings->getFarm(text);
    *p_username = (*p_farm)->getUsername();
    *p_password = (*p_farm)->getPassword();
    *p_domain = (*p_farm)->getCurrentDomain();

    usernameLineEdit->setText((*p_farm)->getUsername());
    domainComboBox->clear();
    domainComboBox->addItems((*p_farm)->getDomainList());
    domainComboBox->setCurrentIndex(domainComboBox->findText(*p_domain));
    domainComboBox->setEditable(true);
}

void ChangePasswordDialog::accept()
{
    QString username = usernameLineEdit->text();
    QString domain = domainComboBox->currentText();
    QString oldPassword = oldPasswordLineEdit->text();
    QString newPassword = newPasswordLineEdit->text();
    QString confirmPassword = confirmPasswordLineEdit->text();

    if (username.isEmpty() || domain.isEmpty() || oldPassword.isEmpty() || newPassword.isEmpty() || confirmPassword.isEmpty())
    {
        QMessageBox::information(this, tr("Attention"), tr("Fields should not be empty"));
        return;
    }

    if (newPassword != confirmPassword)
    {
        QMessageBox::information(this, tr("Attention"), tr("The password doesn't confirm."));
        return;
    }

    *p_farm = p_settings->getFarm(farmNameComboBox->currentText());
    *p_username = usernameLineEdit->text();
    *p_domain = domainComboBox->currentText();
    *p_password = oldPasswordLineEdit->text();
    *p_newPassword = newPasswordLineEdit->text();
    
    QStringList domains;
    for (int i = 0; i < domainComboBox->count(); ++i)
	domains << domainComboBox->itemText(i);
    (*p_farm)->setDomainList(domains);

    return QDialog::accept();
}
