// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include "application.h"
#include <syslog.h>
#include <QtDebug>
#include <QFile>

Application::Application(const QString & responseString)
        : QObject(),
        m_largeIcon(0),
        m_smallIcon(0),
        m_cache(responseString),
        m_enabled(true)
{
    // Getting params list from the responseString
    QStringList appParams = responseString.split(",");
    //qDebug() << responseString;

    // Parsing parameters
    foreach (QString param, appParams)
    {
        // Display Name of application
        if (param.startsWith("ShortCut=", Qt::CaseInsensitive))
        {
            m_displayName = param;
            m_displayName.remove("ShortCut=");
            continue;
        }

        // Name of application
        if (param.startsWith("Name=", Qt::CaseInsensitive))
        {
            m_name = param;
            m_name.remove("Name=");
            continue;
        }

        // Folder of application
        if (param.startsWith("FolderName=", Qt::CaseInsensitive))
        {
            m_folder = param;
            m_folder.remove("FolderName=");
            continue;
        }

        // Small icon for application
        if (param.startsWith("SmallIcon=", Qt::CaseInsensitive))
        {
            QString appSmallIcon = param;
            appSmallIcon.remove("SmallIcon=");
            // Loading small icon from the temp file
            m_smallIcon = fileToPixmap(appSmallIcon);

            // If no valid icon is available, use the default
            if (!m_smallIcon->isNull())
                m_smallIconName = appSmallIcon;
            else
            {
                // Leave the m_smallIconName empty so denote that this is a default icon
                m_smallIcon = new QPixmap( ":/img2/DefaultApp.png");
                syslog( LOG_WARNING, "Invalid icon detected from broker for application '%s'", m_name.toAscii().data());
            }
            continue;
        }

        // Large icon for application
        if (param.startsWith("LargeIcon=", Qt::CaseInsensitive))
        {
            QString appLargeIcon = param;
            appLargeIcon.remove("LargeIcon=");
            // Loading large icon from the temp file
            m_largeIcon = fileToPixmap(appLargeIcon);

            // If no valid icon is available, use the default
            if (!m_largeIcon->isNull())
                m_largeIconName = appLargeIcon;

            else
            {
                // Leave the m_largeIconName empty so denote that this is a default icon
                m_largeIcon = new QPixmap( ":/img2/DefaultApp.png");
                syslog( LOG_WARNING, "Invalid icon detected from broker for application '%s'", m_name.toAscii().data());
            }
            continue;
        }

        //Enable/disable item
        if (param.startsWith("Enabled=", Qt::CaseInsensitive))
        {
            QString enabled = param;
            enabled.remove("Enabled=");
            m_enabled = enabled.startsWith("true", Qt::CaseInsensitive);
            continue;
        }
    }
}

QString Application::displayName() const
{
    return m_displayName;
}

QString Application::name() const
{
    return m_name;
}

QString Application::folder() const
{
    return m_folder;
}

QString Application::cache() const
{
    return m_cache;
}

QPixmap * Application::smallIcon() const
{
    return m_smallIcon;
}

QPixmap * Application::largeIcon() const
{
    return m_largeIcon;
}

void Application::clearCache()
{
    // Clearing temporary application files
    //
    // DCT: Adding check to make sure that the files are valid to begin with
    //
    if ( !m_smallIconName.isNull() && !m_smallIconName.isEmpty() )
    {
        QFile smallIconFile(m_smallIconName);
        smallIconFile.remove();
    }

    if ( !m_largeIconName.isNull()  && !m_smallIconName.isEmpty())
    {
        QFile largeIconFile(m_largeIconName);
        largeIconFile.remove();
    }
}

bool Application::isEnabled()
{
    return m_enabled;
}
