// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#ifndef FARMCONNECTION_H_
#define FARMCONNECTION_H_

#include <QObject>
#include <QProcess>
#include <QStringList>
#include <QApplication>
#include <QStack>

#include <QList>

#include "farm.h"
#include "FarmClient/applicationList.h"
#include "farmSettings.h"

class FarmConnection : public QObject
{
    Q_OBJECT

public:

    enum FinishCode { Reject, Success, Error, Message };
    FarmConnection(QObject * parent = 0);
    virtual ~FarmConnection();

    void setFarm(Farm *farm);
    Farm * farm() const;

    bool connectTo();
    void disconnectTo();

    bool isConnected() const;

    QString lastError() const;

    ApplicationList applications() const;


    bool refreshApplicationsList();
    void executeSelectedApplication(const QString & applicationName);

    bool changeCurrentLocation();
    bool logonAsADifferentUser();
    bool changePassword(FarmSettings *settings);

    void setRequestCredentials(bool requestCreds);

signals:
    void finished(FarmConnection::FinishCode exitCode);
    void error();
    void stateChanged(const QString &);

    void started();
    void startApplication(const QString & name);

    void changePasswordFinished(FarmConnection::FinishCode exitCode, FarmConnection *connection);

public slots:
    void reject();

private slots:
    void runningApplicationFinished(int exitCode, QProcess::ExitStatus exitStatus);
    void refreshingFinished(int exitCode, QProcess::ExitStatus exitStatus);
    void changingPasswordFinished(int exitCode, QProcess::ExitStatus exitStatus);
    void readAvailableDataOnRefreshing();
    void readAvailableDataOnStart();

    void runNextApplication();

private:
    QStringList processRequest();
    QStringList createChangePasswordRequest(const QString & newPassword);

    void changePasswordRequest(const QString & newPassword);
    void setConnected(bool connected);

    QProcessEnvironment getEnvironmentWithUtf8();

    QProcess *pntsc;
    Farm *p_farm;
    QString p_lastError;


    QString p_username;
    QString p_password;
    QString p_domain;

    QStringList p_autoLaunchList;

    bool isAutoLaunched;

    bool p_connected;
    bool p_requestCreds;

    ApplicationList p_applications;
};

#endif /* FARMCONNECTION_H_ */
