// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include "icons.h"
#include <QtDebug>

QPixmap * fileToPixmap(const QString & fileName)
{
	const int SYMBOLS_ON_BYTE = 2;
	const int SYMBOLS_ON_INT = 8;
	const int HEAD_OFFSET = 17;
	const int PICSIZE_BYTE = 5;
	//const int AMASK_BYTE = 9;
	const int RMASK_BYTE = 10;
	const int GMASK_BYTE = 11;
	const int BMASK_BYTE = 12;
	const int WIDTH_BYTE = 6;
	const int HEIGHT_BYTE = 7;

        QPixmap *pixmap = new QPixmap;

	// Try to open
        QFile file(fileName);
        //qDebug() << file.permissions() << file.exists();
	if (!file.open(QIODevice::ReadOnly))
                return pixmap;

	// If size == 0
	if (file.size()== 0)
	{
		return pixmap;
        }

	// Reading file data trimmed
	QByteArray data;
	while (!file.atEnd())
	{
		QByteArray temp = file.readLine();
		data.append(temp.trimmed());
	}
	file.close();

	bool ok;

	// Size of image in pixels
        int imageHeight =
		data.mid(HEIGHT_BYTE * SYMBOLS_ON_INT, SYMBOLS_ON_INT).toUInt(&ok, 16);
	int imageWidth =
                data.mid(WIDTH_BYTE * SYMBOLS_ON_INT, SYMBOLS_ON_INT).toUInt(&ok, 16);

	// Getting masks and shifts
	int blueMask =
		data.mid((HEAD_OFFSET + imageWidth + BMASK_BYTE) * SYMBOLS_ON_INT, SYMBOLS_ON_INT).toUInt(&ok, 16);
	int brShift = maskRightShift(blueMask);
	int blShift = maskLeftShift(blueMask);

	int greenMask =
		data.mid((HEAD_OFFSET + imageWidth + GMASK_BYTE) * SYMBOLS_ON_INT, SYMBOLS_ON_INT).toUInt(&ok, 16);
	int grShift = maskRightShift(greenMask);
	int glShift = maskLeftShift(greenMask);

	int redMask =
		data.mid((HEAD_OFFSET + imageWidth + RMASK_BYTE) * SYMBOLS_ON_INT, SYMBOLS_ON_INT).toUInt(&ok, 16);
	int rrShift = maskRightShift(redMask);
        int rlShift = maskLeftShift(redMask);

	// Getting image size
	int imageSize =
		data.mid((HEAD_OFFSET + imageWidth + PICSIZE_BYTE) * SYMBOLS_ON_INT, SYMBOLS_ON_INT).toUInt(&ok, 16);

	// Calculating value byte per pixel
	int bytesPerPixel = imageSize / (imageHeight * imageWidth);

	// Getting image data
	QByteArray imageData = data.right(imageSize * SYMBOLS_ON_BYTE);

	// Create out picture
	QImage image(imageWidth, imageHeight, QImage::Format_RGB32);
	image.fill(Qt::transparent);

	// Fill pixels in this picture
	for (int i = 0; i < imageHeight; i++)
		for (int j = 0; j < imageWidth; j++)
		{
			// Starting offset reading bytes
			int offset = (j * imageWidth + i) * bytesPerPixel * SYMBOLS_ON_BYTE;
			// Getting cross over bytes of our color
			QByteArray colorArray;
			for (int counter = 0; counter < bytesPerPixel; counter++)
			{
				colorArray.prepend(imageData.mid(offset + counter * SYMBOLS_ON_BYTE,
						SYMBOLS_ON_BYTE));
			}

			// Color value for this pixel
			int color = colorArray.toUInt(&ok, 16);

			int r = ((color & redMask) >> rrShift) << rlShift;
			int g = ((color & greenMask) >> grShift) << glShift;
			int b = ((color & blueMask) >> brShift) << blShift;

			image.setPixel(i, imageWidth - j - 1, qRgb(r, g, b));
		}

	*pixmap = QPixmap::fromImage(image);
	pixmap->setMask(pixmap->createMaskFromColor(Qt::black));

	return pixmap;
}

int maskRightShift(int mask)
{
	// Translating number to bin string
	QString binMask = QString::number(mask, 2);
	int binMaskSize = binMask.length();

	// Getting right numbers
	int lastOneIndex = binMask.lastIndexOf('1');
	if (lastOneIndex < 0)
		return binMaskSize;

	return binMaskSize - lastOneIndex - 1;
}

int maskLeftShift(int mask)
{
	// Translating number to bin string
	QString binMask = QString::number(mask, 2);

	// Getting one string
	int firstOneIndex = binMask.indexOf('1');
	int lastOneIndex = binMask.lastIndexOf('1');

	if (firstOneIndex < 0 || lastOneIndex < 0)
		return 0;

	return 8 - (lastOneIndex - firstOneIndex + 1);
}

