// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include <stdexcept>
#include "autoLaunchPage.h"


AutoLaunchPage::AutoLaunchPage(Farm* farm, ManageConnectionDialog* parent)
    : ManageConnectionPage(farm, parent)
{
    setupUi(this);

    bool connected = true;
    connected &= connect(donePushButton, SIGNAL(clicked()), parent, SLOT(accept()));
    connected &= connect(addButton, SIGNAL(clicked()), SLOT(addApplication()));
    connected &= connect(deletePushButton, SIGNAL(clicked()), SLOT(removeApplication()));
    connected &= connect(upButton, SIGNAL(clicked()), SLOT(upApplication()));
    connected &= connect(downButton, SIGNAL(clicked()), SLOT(downApplication()));
    if (!connected)
    {
        throw std::runtime_error("QT's signal and slot connection error in AutoLaunchPage constructor");
    }

    loadFarmSettings();
}

void AutoLaunchPage::loadFarmSettings()
{
    // Farm doesn`t exist
    if (m_pCurrentFarm == NULL)
    {
        return;
    }

    applicationsComboBox->clear();
    ApplicationList apps = m_pCurrentFarm->applications();
    foreach (Application* app, apps)
    {
        // Only do this if the icon is not empty
        if (app->smallIcon() != NULL)
        {
            applicationsComboBox->addItem(*(app->smallIcon()), app->name());
        }
    }

    addButton->setEnabled(!apps.isEmpty());

    autolaunchListWidget->clear();
    ApplicationList autolaunch = m_pCurrentFarm->autolaunch();
    QStringList autolaunchStringList = autolaunch.names();
    foreach (QString appName, autolaunchStringList)
    {
        Application* app = apps.application(appName);
        if (app != NULL)
        {
            QListWidgetItem* item = new QListWidgetItem;
            item->setIcon(*(app->smallIcon()));
            item->setText(app->name());
            autolaunchListWidget->addItem(item);
        }
    }
   autoLaunchOneAppCheckBox->setChecked(m_pCurrentFarm->autoConnectIfOneApplication());
}

void AutoLaunchPage::saveFarmSettings()
{
    // Farm doesn`t exist
    if (m_pCurrentFarm == NULL)
    {
        return;
    }

    ApplicationList apps = m_pCurrentFarm->applications();
    ApplicationList autolaunch;
    for (int i = 0; i < autolaunchListWidget->count(); ++i)
    {
        QString name = autolaunchListWidget->item(i)->text();
        Application* app = apps.application(name);
        if (app != NULL)
        {
            autolaunch << app;
        }
    }
    m_pCurrentFarm->setAutolaunch(autolaunch);
    m_pCurrentFarm->setAutoConnectIfOneApp(autoLaunchOneAppCheckBox->isChecked());
}

void AutoLaunchPage::addApplication()
{
    QString addedApplicationName = applicationsComboBox->currentText();
    if (!addedApplicationName.isEmpty() && autolaunchListWidget->findItems(addedApplicationName, Qt::MatchFixedString).isEmpty())
    {
        QIcon addedApplicationicon = applicationsComboBox->itemIcon(applicationsComboBox->currentIndex());
        QListWidgetItem* newItem = new QListWidgetItem(addedApplicationicon, addedApplicationName);
        autolaunchListWidget->addItem(newItem);
    }
}

void AutoLaunchPage::removeApplication()
{
    if (autolaunchListWidget->currentItem() == NULL)
    {
        return;
    }
    autolaunchListWidget->takeItem(autolaunchListWidget->currentRow());
}

void AutoLaunchPage::upApplication()
{
    int currentIndex = autolaunchListWidget->currentRow();
    if (currentIndex > 0)
    {
        QListWidgetItem *tempItem = autolaunchListWidget->takeItem(currentIndex);
        autolaunchListWidget->insertItem(currentIndex - 1, tempItem);
        autolaunchListWidget->setCurrentItem(tempItem);
    }
    autolaunchListWidget->setFocus();
}

void AutoLaunchPage::downApplication()
{
    int currentIndex = autolaunchListWidget->currentRow();
    if (currentIndex < autolaunchListWidget->count())
    {
        QListWidgetItem *tempItem = autolaunchListWidget->takeItem(currentIndex);
        autolaunchListWidget->insertItem(currentIndex + 1, tempItem);
        autolaunchListWidget->setCurrentItem(tempItem);
    }
    autolaunchListWidget->setFocus();
}
