// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include <stdexcept>
#include "ManageConnectionDialog/manageConnectionDialog.h"
#include "ShowStepsDialog/showStepsDialog.h"
#include "helpWindow.h"


HelpWindow::HelpWindow(QWidget* parent)
    : CommonDialog(parent)
{
    if (!parent)
    {
        throw std::runtime_error("Parameter \'parent\' is null in HelpWindow constructor");
    }

    Ui::HelpWindow::setupUi(this);
    setWindowMask(":/img/mask_with_arrow.png", 0, 0);

    m_pParentDialog = static_cast<ManageConnectionDialog*>(parent);

    bool connected = true;
    connected &= connect(backButton, SIGNAL(clicked()), parent, SLOT(back()));
    connected &= connect(nextButton, SIGNAL(clicked()), parent, SLOT(next()));
    connected &= connect(closeButton, SIGNAL(clicked()), SLOT(closeClicked()));
    if (!connected)
    {
        throw std::runtime_error("QT's signal and slot connection error in HelpWindow constructor");
    }

    m_pShowStepsDialog = new ShowStepsDialog(parent);
}

void HelpWindow::setStepText(const QString& step)
{
    stepLabel->setText(step);
}

void HelpWindow::setHelpText(const QString& help)
{
    helpLabel->setText(help);
}

void HelpWindow::setBackButtonVisible(bool visible)
{
    backButton->setVisible(visible);
}

void HelpWindow::setNextButtonVisible(bool visible)
{
    nextButton->setVisible(visible);
}

void HelpWindow::closeClicked()
{
    const bool showStepsAccepted = m_pShowStepsDialog->exec() == QDialog::Accepted;
    m_pParentDialog->setShowStepsNextTime(showStepsAccepted);
    close();
}
