// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include <stdexcept>
#include "farmSettingsNames.h"
#include "localResourcesPage.h"


const QString localAudioMode =  "local";
const QString remoteAudioMode = "remote";
const QString offAudioMode =    "off";
const int localAudioIndex =  0;
const int remoteAudioIndex = 1;
const int offAudioIndex =    2;


LocalResourcesPage::LocalResourcesPage(Farm* farm, ManageConnectionDialog* parent)
    : ManageConnectionPage(farm, parent)
{
    setupUi(this);
    microphoneCheckBox->hide(); // This option is hidden in 7.6 release according to a customer request.
    bool connected = connect(donePushButton, SIGNAL(clicked()), parent, SLOT(accept()));
    connected &= connect(microphoneCheckBox, SIGNAL(toggled(bool)), SLOT(microphoneToggled(bool)));
    if (!connected)
    {
        throw std::runtime_error("QT's signal and slot connection error in LocalResourcesPage constructor");
    }

    loadFarmSettings();
}

void LocalResourcesPage::loadFarmSettings()
{
    // Farm doesn`t exist
    if (m_pCurrentFarm == NULL)
    {
        return;
    }

    QString soundMode = m_pCurrentFarm->getSoundMode();
    setSoundComboBoxIndex(soundMode);

    diskDriversCheckBox->setChecked(m_pCurrentFarm->isRedirectDrives());
    printersCheckBox->setChecked(m_pCurrentFarm->isRedirectPrinters());
    usbCheckBox->setChecked(m_pCurrentFarm->isRedirectHandhelds());
    serialPortsCheckBox->setChecked(m_pCurrentFarm->isRedirectComPorts());
    clipboardCheckBox->setChecked(m_pCurrentFarm->isRedirectClipboard());
    microphoneCheckBox->setChecked(m_pCurrentFarm->isRedirectMicrophone());
    smartCardCheckBox->setChecked(m_pCurrentFarm->isRedirectSmartCards());

    QString tmpStr;
    bool tmpBool;

    soundComboBox->setEnabled(!m_appSettings->getValue(REDIRECT_AUDIO, &tmpStr));

    diskDriversCheckBox->setEnabled(!m_appSettings->getValue(REDIRECT_DRIVES, &tmpBool));
    printersCheckBox->setEnabled(!m_appSettings->getValue(REDIRECT_PRINTERS, &tmpBool));
    usbCheckBox->setEnabled(!m_appSettings->getValue(REDIRECT_HANDHELDS, &tmpBool));
    serialPortsCheckBox->setEnabled(!m_appSettings->getValue(REDIRECT_COM_PORTS, &tmpBool));
    clipboardCheckBox->setEnabled(!m_appSettings->getValue(REDIRECT_CLIPBOARD, &tmpBool));
    microphoneCheckBox->setEnabled(!m_appSettings->getValue(REDIRECT_MICROPHONE, &tmpBool));

    // For now "Apply Windows Key Combinations" is not loaded
    // becouse of absense of appropriate field in Farm
}

void LocalResourcesPage::saveFarmSettings()
{
    // Farm doesn`t exist
    if (m_pCurrentFarm == NULL)
    {
        return;
    }

    int audioModeIndex = soundComboBox->currentIndex();
    QString soundMode;
    switch (audioModeIndex)
    {
    case localAudioIndex:
        soundMode = localAudioMode;
        break;
    case remoteAudioIndex:
        soundMode = remoteAudioMode;
        break;
    default:
        soundMode = offAudioMode;
    }
    m_pCurrentFarm->setSoundMode(soundMode);

    m_pCurrentFarm->setRedirectDrives(diskDriversCheckBox->isChecked());
    m_pCurrentFarm->setRedirectPrinters(printersCheckBox->isChecked());
    m_pCurrentFarm->setRedirectHandhelds(usbCheckBox->isChecked());
    m_pCurrentFarm->setRedirectComPorts(serialPortsCheckBox->isChecked());
    m_pCurrentFarm->setRedirectClipboard(clipboardCheckBox->isChecked());
    m_pCurrentFarm->setRedirectMicrophone(microphoneCheckBox->isChecked());
    m_pCurrentFarm->setRedirectSmartCards( smartCardCheckBox->isChecked());

    // For now "Apply Windows Key Combinations" is not saved
    // becouse of absense of appropriate field in Farm

}

void LocalResourcesPage::eopAudioToggled(bool checked)
{
    if (checked)
    {
        microphoneCheckBox->setChecked(true);
        soundComboBox->setCurrentIndex(localAudioIndex);
        soundComboBox->setEnabled(false);
        microphoneCheckBox->setEnabled(false);
    }
    else
    {
        QString soundMode;
        soundComboBox->setEnabled(!m_appSettings->getValue(REDIRECT_AUDIO, &soundMode));
        setSoundComboBoxIndex(soundMode);
        microphoneCheckBox->setEnabled(true);
    }
}

void LocalResourcesPage::microphoneToggled(bool checked)
{
    if (checked)
    {
        soundComboBox->setCurrentIndex(localAudioIndex);
    }
}

void LocalResourcesPage::setSoundComboBoxIndex(const QString& soundMode)
{
    int audioModeIndex = offAudioIndex;
    if (soundMode == localAudioMode)
    {
        audioModeIndex = localAudioIndex;
    }
    if (soundMode == remoteAudioMode)
    {
        audioModeIndex = remoteAudioIndex;
    }
    soundComboBox->setCurrentIndex(audioModeIndex);
}
