// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#ifndef LOCATION_DIALOG_H
#define LOCATION_DIALOG_H

#include <QSharedPointer>
#include <QSslSocket>
#include "common/applicationSettings.h"
#include "commonDialog.h"
#include "ManageConnectionDialog/AddBrokerDialog/addBrokerDialog.h"
#include "ui_locationDialog.h"

class Farm;
class AddBrokerDialog;
typedef QSharedPointer<ApplicationSettings> SharedApplicationSettings;
typedef QSharedPointer<QSslSocket> SocketSharedPointer;

const QString httpProtocol  = "http";
const QString httpsProtocol = "https";


class LocationDialog : public CommonDialog,
    private Ui::LocationDialog
{
    Q_OBJECT

public:
    LocationDialog(Farm* farm, QWidget* parent = 0);

    void loadFarmSettings();
    void saveFarmSettings();

private slots:
    void acceptChanges();
    void currentProxyOptionChanged(QAbstractButton* button);
    void enableSslGateway(bool enableSslGateway);
    void upBroker();
    void downBroker();
    void removeBroker();
    void addBroker();
    void testBroker();
    void testBrokerState(QAbstractSocket::SocketState state);
    void checkSelectedBroker();
    void currentProtocolChanged(const QString& protocol);

private:
    void setEnabledBrokerButtons(bool enabled);

private:
    Farm* m_pCurrentFarm;
    SharedApplicationSettings m_appSettings;
    QButtonGroup* m_pProxyButtonGroup;
    AddBrokerDialog* m_pAddBrokerDialog;
    SocketSharedPointer m_pSocket;
};

#endif // LOCATION_DIALOG_H
