// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#ifndef MANAGECONNECTIONDIALOG_H
#define MANAGECONNECTIONDIALOG_H

#include <QtGui/QDialog>
#include <QStackedWidget>
#include "farmSettings.h"
#include "ui_manageConnectionDialog.h"

class HelpWindow;
class DeleteFarmDialog;
class LocalResourcesPage;


class ManageConnectionDialog : public QDialog,
    private Ui::ManageConnectionDialogClass
{
    Q_OBJECT

public:
    ManageConnectionDialog(FarmSettings* settings, bool isAddFarmMode,
                           const QString& actuallyFarmName, QWidget* parent = 0);
    ~ManageConnectionDialog();

    // === GUI functions
    void addPage(QWidget* page, const QString& icon);

    // === Functionality
    FarmSettings* getFarmSettings() { return m_pFarmSettings; };
    void saveFarmSettings();
    void setShowStepsNextTime(bool showNextTime);
    void setAllFarmsNotAutoConnected();

public slots:
    // === GUI functions
    void back();
    void next();
    void eopAudioToggled(bool checked);

    // === Functionality
    void loadFarmSettings();

protected:
    // === GUI functions
    void closeEvent(QCloseEvent* event);

private slots:
    // === GUI functions
    void accept();
    void cancel();
    void switchToBasicView();
    void currentPageChanged(int index);

    // === Functionality
    void deleteFarm();

private:
    // === GUI functions
    void setDialogMode(bool isAddFarmMode);
    void createHelpWindow();
    void createConnectionDialog();
    void moveHelpWindow();
    void mousePressEvent(QMouseEvent* event);
    void mouseMoveEvent(QMouseEvent* event);

    // === Functionality
    void setFarmForAddDialog(const QString& name = "");
    void setFarmForEditDialog(const QString& name = "");
    bool isFarmLocationsCorrect(Farm* farm);
    void updateNavButtons();

    QPoint m_dragPosition;

    FarmSettings* m_pFarmSettings;
    Farm* m_pCurrentFarm;
    QString m_currentFarmName;

    bool m_isAddFarmMode; // There are Add Farm and Edit Farm modes
    bool m_deleteCurrentFarmOnClose;

    HelpWindow* m_pHelpWindow;
    QVector<QString> m_helpString;

    DeleteFarmDialog* m_pDeleteFarmDialog;
    LocalResourcesPage* m_pLocalResourcesPage;
};

#endif // MANAGECONNECTIONDIALOG_H
