// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include "usbrwidget.h"
#include "ui_usbrwidget.h"
#include "QMessageBox"

UsbrDeviceTwi::UsbrDeviceTwi(QTreeWidget *p)
    :QTreeWidgetItem(p)
{
   return;
};

UsbrWidget::UsbrWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::UsbrWidget)
{
    ui->setupUi(this);
    program = "/usr/local/bin/usbsrv";
    list();
    info();
}

UsbrWidget::~UsbrWidget()
{
    delete ui;
}

int UsbrWidget::execute_program(QStringList arguments)
{
    usbr_process = new QProcess(this);
    usbr_process->start(program, arguments);
    usbr_process->waitForFinished();
    usbr_stdout = usbr_process->readAllStandardOutput();
    return 0;
}

void UsbrWidget::on_share_pushButton_released()
{

    UsbrDeviceTwi* tmp = (UsbrDeviceTwi*)(ui->treeWidget->currentItem());
    QStringList arguments;
    arguments << "-share" << tmp->idStr;
    execute_program(arguments);
    list();
}

void UsbrWidget::on_unshare_pushButton_released()
{
    UsbrDeviceTwi* tmp = (UsbrDeviceTwi*)(ui->treeWidget->currentItem());
    QStringList arguments;
    arguments << "-unshare" << tmp->idStr;
    execute_program(arguments);
    list();
}

void UsbrWidget::on_exclude_pushButton_released()
{
    UsbrDeviceTwi* tmp = (UsbrDeviceTwi*)(ui->treeWidget->currentItem());
    QStringList arguments;
    arguments << "-addexclude" << tmp->idStr;
    execute_program(arguments);
    list();
}

void UsbrWidget::on_unexclude_pushButton_released()
{
    UsbrDeviceTwi* tmp = (UsbrDeviceTwi*)(ui->treeWidget->currentItem());
    QStringList arguments;
    arguments << "-remexclude" << tmp->idStr;
    execute_program(arguments);
    list();
}

void UsbrWidget::on_autoshare_checkBox_stateChanged(int arg1)
{
    QStringList arguments;
    if(arg1)
    {
       arguments << "-autoshareon";
    }
    else
    {
       arguments << "-autoshareoff";
    }
    execute_program(arguments);
    info();
}

int UsbrWidget::list()
{
    ui->treeWidget->clear();
    ui->share_pushButton->setEnabled(FALSE);
    ui->unshare_pushButton->setEnabled(FALSE);
    ui->exclude_pushButton->setEnabled(FALSE);
    ui->unexclude_pushButton->setEnabled(FALSE);

    QStringList arguments;
    arguments << "-list";
    execute_program(arguments);
    QString output(usbr_stdout);
    if(output.contains("ERROR"))
    {
        QMessageBox::critical(this, "usb redirector error", "There is a error communicating with the usb redirector daemon, please check if it is running properly");
        return 1;
    }
    QStringList strList = output.split("\n",QString::SkipEmptyParts);
/*
    output.append("++++++++++++ PARSE1 ++++++++++++++\n");
    for(int i = 0; i < strList.size(); ++i)
    {
        output.append("[");
        output.append(strList.at(i).simplified());
        output.append("]\n");
    }
*/

    for(int i = 3; i+3 < strList.size(); i+=3)
    {

        UsbrDeviceTwi* tmp = new UsbrDeviceTwi(ui->treeWidget);


       bool ok;
       QStringList part1 = strList.at(i).split(":");
       QStringList part2 = strList.at(i+1).split(":");
       QStringList part3 = strList.at(i+2).split(":");

       tmp->id       = part1.at(0).toInt(&ok);
       tmp->idStr    = part1.at(0);
       tmp->name     = part1.at(1);
       tmp->pci_pid  = part2.at(1).simplified();
       tmp->pci_pid.truncate(4);
       tmp->pci_vid  = part2.at(2).simplified();
       tmp->pci_vid.truncate(4);
       tmp->port     = part2.at(3);
       tmp->shared   = part3.at(1).contains("shared");
       tmp->excluded = part3.at(1).contains("exclusion");
       tmp->plugged  = part3.at(1).contains("plugged");

       tmp->setText(0,tmp->name);
       QString tmpstr = tmp->pci_pid;
       tmpstr.append(":");
       tmpstr.append(tmp->pci_vid);
       tmp->setText(1, tmpstr);
       tmp->setText(2,tmp->port);
       tmpstr.clear();
       if(tmp->shared)   tmpstr.append("shared ");
       if(tmp->excluded) tmpstr.append("excluded ");
       if(tmp->plugged)  tmpstr.append("plugged ");
       tmp->setText(3,tmpstr);
    }
    return 0;
}

int UsbrWidget::listClients()
{
    QStringList arguments;
    arguments << "-listclients";
    execute_program(arguments);
    QString output(usbr_stdout);
    //ui->infoTextEdit->setText(output);
    return 0;
}

int UsbrWidget::info()
{
    //    USB Server information:
    //    ======================
    //    Total USB devices in configuration: 0
    //    Total connected clients: 0
    //    USB device sharing mode: manual-sharing
    //    USB Server accepts incoming connections on 32032 tcp port
    QStringList arguments;
    arguments << "-info";
    execute_program(arguments);
    QString output(usbr_stdout);
    if(output.contains("ERROR"))
    {
        QMessageBox::critical(this, "usb redirector error", "There is a error communicating with the usb redirector daemon, please check if it is running properly");
        return 1;
    }

    if(output.contains("manual-sharing"))
    {
        ui->autoshare_checkBox->setEnabled(TRUE);
        ui->autoshare_checkBox->setCheckState(Qt::Unchecked);
    }
    else if(output.contains("auto-sharing"))
    {
        ui->autoshare_checkBox->setEnabled(TRUE);
        ui->autoshare_checkBox->setCheckState(Qt::Checked);
    }
    else
    {
        QMessageBox::critical(this,"usb redirector error", "unable to determine auto-sharing state");
        ui->autoshare_checkBox->setCheckState(Qt::Unchecked);
        ui->autoshare_checkBox->setEnabled(FALSE);
    }

    //    ui->infoTextEdit->setText(output);

   return 0;
}

void UsbrWidget::on_treeWidget_itemSelectionChanged()
{
    UsbrDeviceTwi* tmp = (UsbrDeviceTwi*)(ui->treeWidget->currentItem());
    if(tmp->shared)
    {
        ui->share_pushButton->setEnabled(FALSE);
        ui->unshare_pushButton->setEnabled(TRUE);
    }
    else
    {
        ui->share_pushButton->setEnabled(TRUE);
        ui->unshare_pushButton->setEnabled(FALSE);
    }

    if(tmp->excluded)
    {
        ui->exclude_pushButton->setEnabled(FALSE);
        ui->unexclude_pushButton->setEnabled(TRUE);
    }
    else
    {
        ui->exclude_pushButton->setEnabled(TRUE);
        ui->unexclude_pushButton->setEnabled(FALSE);
    }
}

void UsbrWidget::refresh()
{
    list();
    info();
}

void UsbrWidget::on_pushButton_released()
{
    list();
    info();
}
