// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#ifndef APPPORTAL_CONFIG_COLLECTOR
#define APPPORTAL_CONFIG_COLLECTOR

#include <QObject>
#include <QString>
#include <QNetworkReply>
#include <QMap>
#include <QSharedPointer>
#include <QByteArray>

enum ConfigResult {
    CONFIG_OK,
    CONFIG_DIR_IS_EMPTY,
    CONFIG_FILE_NAME_IS_EMPTY,
    CONFIG_FILE_DOES_NOT_EXIST,
    CONFIG_FILE_COULD_NOT_BE_READ,
    CONFIG_URL_IS_EMPTY,
    CONFIG_WRONG_PROXY_PORT
};


class QNetworkManager;

typedef QSharedPointer<QByteArray> SharedBuffer;
typedef QMap<QString, SharedBuffer> ConfigMap; // QMap<pathToConfig, sharedBufferOfConfig>
typedef QMap<QString, SharedBuffer>::iterator ConfigMapIterator;
typedef QMap<QString, SharedBuffer>::const_iterator ConfigMapConstIterator;


class AppPortalConfigCollector : public QObject
{

    Q_OBJECT

public:
    AppPortalConfigCollector(QObject* parent = 0);
    ~AppPortalConfigCollector();
    ConfigResult loadConfigFromFile(QString dir, QString fileName);
    ConfigResult loadConfigFromNetwork(QString protocol, QString url, QString fileName, bool isProxy, QString proxyList);
    SharedBuffer getConfigBuffer(QString protocol, QString url, QString fileName) const;
    SharedBuffer getConfigBuffer(QString fullUrl) const;
    bool deleteConfig(QString protocol, QString url, QString fileName);
    bool deleteConfig(QString fullUrl);
    void deleteAll();

signals:
    void networkReply(QNetworkReply::NetworkError error, QString errorString, QString urlString);

private slots:
    void replyFinished(QNetworkReply* reply);

private:
    QString makePathToConfig(QString& protocol, QString& url, QString& fileName) const;

    ConfigMap m_configMap;
    QNetworkAccessManager* m_pNetworkManager;
};

#endif
