// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include <QApplication>
#include <QPalette>
#include <QBrush>
#include <QDir>
#include <QAction>
#include "applicationListModel.h"
#include "FarmClient/applicationList.h"
#include "FarmClient/application.h"
#include "sublib/icons.h"
#include <syslog.h>


ApplicationListModel::ApplicationListModel(QObject* parent)
    : QStandardItemModel(parent)
{
}

void ApplicationListModel::setApplicationList(ApplicationList applicationList)
{
    m_fullModel.clear();

    if ( applicationList.names().contains( "No available applications" ) )
    {
        syslog( LOG_WARNING, "The broker has return an empty application list.");

        // Clear out the visible list because loading an empty app list is valid
        // and should not throw a error (warning maybe).
        clear();
        return;
    }

    foreach (Application* app, applicationList)
    {
        // Folder of application
        QStandardItem* parentItem = m_fullModel.invisibleRootItem();
        QStringList appFolderPath = app->folder().split("//", QString::SkipEmptyParts);
        QString appFolder = appFolderPath.back();
        if (!appFolder.isEmpty())
        {
            QList<QStandardItem*> existingFolders = m_fullModel.findItems(appFolder, Qt::MatchExactly);
            if (existingFolders.isEmpty())
            {
                QStandardItem* newFolderItem = new QStandardItem(QIcon(":/img2/folder.png"), appFolder);
                newFolderItem->setData(FolderItemType, ItemTypeRole);
                newFolderItem->setData(appFolder, Qt::DisplayRole);
                parentItem->appendRow(newFolderItem);
                QStandardItem* newBackItem = new QStandardItem(QIcon(":/img2/folder_back.png"), tr("ApplicationList.Back..."));
                newBackItem->setData(BackItemType, ItemTypeRole);
                newFolderItem->appendRow(newBackItem);
                parentItem = newFolderItem;
            }
            else
            {
                parentItem = existingFolders.first();
            }
        }
        QStandardItem* newAppItem = new QStandardItem(*(app->largeIcon()), app->displayName().isEmpty() ? app->name() : app->displayName());
        newAppItem->setData(ApplicationItemType, ItemTypeRole);
        newAppItem->setData(applicationList.indexOf(app), ItemIndexRole );
        parentItem->appendRow(newAppItem);
    }

    showRoot();
}

void ApplicationListModel::showRoot()
{
    clear();
    QStandardItem* currentItem = m_fullModel.invisibleRootItem();
    int itemsCount = currentItem->rowCount();
    for (int i = 0; i < itemsCount; ++i)
    {
        invisibleRootItem()->appendRow(currentItem->child(i, 0)->clone());
    }
}

void ApplicationListModel::openFolder(int folderIndex)
{
    if (folderIndex < 0)
    {
        return;
    }

    clear();
    QStandardItem* currentItem = m_fullModel.item(folderIndex, 0);
    int itemsCount = currentItem->rowCount();
    for (int i = 0; i < itemsCount; ++i)
    {
        invisibleRootItem()->appendRow(currentItem->child(i, 0)->clone());
    }
}

QString ApplicationListModel::getName(int folderIndex)
{
   QStandardItem* item = m_fullModel.item(folderIndex, 0);

   return item->text();
}
