// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#ifndef _LOCATION_H_
#define _LOCATION_H_

#include <QString>
#include <QStringList>
#include <QDebug>
#include <QSharedPointer>
#include "common/applicationSettings.h"

typedef QSharedPointer<ApplicationSettings> SharedApplicationSettings;


class Location
{
public:
    Location();
    virtual ~Location();

    QString getName() { return name; }
    void setName(const QString& n) { name = n; }
    QString  getPort() { return port; }
    void setPort(const QString& p);
    QString getProtocol() { return protocol; }
    void setProtocol(const QString& p);
    QStringList getServers() { return servers; }
    void setServers(QStringList srvs);
    bool isNat() { return nat; }
    void setNat(bool isEnabled);
    bool isSsl() { return ssl; }
    void setSsl(bool isEnabled);
    QString  getSslGateway() { return sslGateway; }
    void setSslGateway(const QString& gateway);

    // Proxy settings
    QString getProxyServer() {return proxy; }
    void setProxyServer(const QString& pxy);
    QString getProxyBypassList() {return proxyBypassList; }
    void setProxyBypassList(const QString& list);

private:
    QString port;
    QString protocol;
    QStringList servers;
    QString sslGateway;
    bool ssl;
    bool nat;
    QString name;

    QString proxy;
    QString proxyBypassList;

    SharedApplicationSettings m_appSettings;

    template <typename Type>
    void setDefaultValue(const QString& key, Type* variable, const Type value);

    template <typename Type>
    void setValue(const QString& key, Type* variable, const Type value);

    template <typename Type>
    void setValueList(const QString& key, QList<Type>* list, const QList<Type> valueList);
};

#endif
