// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include <QApplication>
#include <QTimer>
#include <QtPlugin>
#include <qmessagebox.h>

#include <cstdio>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <QString>
#include <syslog.h>

#include "mainWindow.h"
#include "../common/common.h"
#include "../qrdesktop/libpopt/output/include/popt.h"
// Allow the __QT_STATIC macro to control the inclusion of the
// plugin directives.
#ifdef __QT_STATIC
Q_IMPORT_PLUGIN( qgif )
Q_IMPORT_PLUGIN( qjpeg )
#endif

MainWindow* MainWindow::m_instance = 0;

#define LOCKFILENAME        "/tmp/app_portal_lock"

static int fdlock;

int get_lock(void)
{
    struct flock fl;

    fl.l_type = F_WRLCK;
    fl.l_whence = SEEK_SET;
    fl.l_start = 0;
    fl.l_len = 1;

    if((fdlock = open( LOCKFILENAME, O_WRONLY|O_CREAT, 0666)) == -1)
        return 0;

    if(fcntl(fdlock, F_SETLK, &fl) == -1)
        return 0;

    return 1;
}

void remove_lock(void)
{
    close(fdlock);
    remove(LOCKFILENAME);
}


int main(int argc, char *argv[])
{
    int rc, rc2;

    Q_INIT_RESOURCE(allresource);

    QApplication app(argc, argv);

    // app.setStyle("motif");

    QTranslator appTranslator;


    bool isTranslationLoaded = appTranslator.load("AppPortal_en", QString( GUI_TRANSLATION_FILE_PATH ));
    if (isTranslationLoaded)
    {
        app.installTranslator(&appTranslator);
    }

    rc = get_lock();


 #ifdef _ZEROPC

    // Fix the font size if using zeropc
    QFont font = qApp->font();
    font.setPointSize( 12 );
    qApp->setFont(font);

#endif

    if ( rc != 0 )
    {
        MainWindow *window = MainWindow::instance();

        // Parsing program options from command line
        int res;
        int isRemoveFarms = 0;
        poptContext optCon;   /* context for parsing command-line options */
        struct poptOption optionsTable[] =
        {
            { "removefarms", 'r', POPT_ARG_NONE, &isRemoveFarms, 0, "remove farms settings from GUI on exit" },
            POPT_TABLEEND
        };
        optCon = poptGetContext(NULL, argc, (const char**)argv, optionsTable, 0);
        res = poptGetNextOpt(optCon);
        if (res == -1)
        {
            // Here errors in command line options can be processed
        }
        poptFreeContext(optCon);
        // Setting the flag that shows if we want to remove farms settings from GUI on exit
        window->setRemoveFarms(isRemoveFarms);

        // Throw a little banner into the messages log to announce the version and build
        syslog( LOG_NOTICE, "===========================================================" );
        syslog( LOG_NOTICE, "Starting: %s", argv[0]);
        syslog( LOG_NOTICE, "Built   : %s %s", __DATE__, __TIME__);
        syslog( LOG_NOTICE, "Version : %s %s", APP_VERSION_FULL_STRING, APP_BUILD_STRING );

#ifdef _ZEROPC
        syslog( LOG_NOTICE, "Platform: ZeroPC" );
#else
        syslog( LOG_NOTICE, "Platform: Standard" );
#endif
        syslog( LOG_NOTICE, "===========================================================" );

        window->show();
        QTimer::singleShot(0, window, SLOT(showStartupTooltips()));
        rc2 = app.exec();

        remove_lock();

        // Throw a little banner into the messages log to announce the version and build
        syslog( LOG_NOTICE, "===========================================================" );
        syslog( LOG_NOTICE, "Exiting: %s", argv[0]);
        syslog( LOG_NOTICE, "===========================================================" );

        return rc2;
    }
    else
    {
        // The lock file is still present, so cannot run, instead, throw the
        // error and display the test in a message window
        QMessageBox::warning( 0, "Warning", "Another instance of this application is already running", QMessageBox::Ok, QMessageBox::NoButton );

        // Throw a little banner into the messages log to announce the version and build
        syslog( LOG_NOTICE, "===========================================================" );
        syslog( LOG_NOTICE, "Aborting: %s", argv[0]);
        syslog( LOG_NOTICE, "===========================================================" );

        return -2;
    }
}
