// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include "minimalviewdialog.h"
#include "ui_minimalviewdialog.h"
#include <syslog.h>
#include "UsbrDialog/usbrdialog.h"

MinimalViewDialog::MinimalViewDialog(QWidget *parent) :
    CommonDialog(parent),
    ui(new Ui::MinimalViewDialog)
{
    ui->setupUi(this);
    // setWindowFlags(Qt::FramelessWindowHint );

    setObjectName( QString( "MinimalViewDialog"));

    setWindowIconText( tr("vWorkspace") );

    // This area was hidden to prevent some issues (unknown for now).
    // Now area is shown to discover is there still issue, and if it is - what exactly is this issue
    // ui->activeConnectionsVerticalFrame->hide();

}

MinimalViewDialog::~MinimalViewDialog()
{
    // syslog( LOG_INFO, "Deleting the Minimal View Dialog");
    delete ui;
    // syslog( LOG_INFO, "Deleted the Minimal View Dialog");
}

void MinimalViewDialog::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MinimalViewDialog::mousePressEvent(QMouseEvent* event)
{
    if (event->button() == Qt::LeftButton)
    {
        m_dragPosition = event->globalPos() - frameGeometry().topLeft();
        event->accept();
    }
}

void MinimalViewDialog::mouseMoveEvent(QMouseEvent* event)
{
    if (event->buttons() & Qt::LeftButton)
    {

        move(event->globalPos() - m_dragPosition);

        // Move the main window too, even if it hidden
        QWidget * mw = (QWidget *) parent();
        mw->move(event->globalPos() - m_dragPosition);

        event->accept();
    }
}


void MinimalViewDialog::setApplicationListModel( ApplicationListModel *  model )
{
    syslog( LOG_INFO, "Assigning Application Model to minimal view");
    ui->minimalViewApplicationListView->setModel( model );
    ui->minimalViewApplicationListView->update();
}


void MinimalViewDialog::addSession( QString name )
{
    syslog( LOG_INFO, "Adding session %s to minimal view", name.toAscii().data());

    // May the layout
    QHBoxLayout * frame1Layout = new QHBoxLayout;
    frame1Layout->setSpacing( 3 );
    frame1Layout->setContentsMargins( 0, 0, 0, 0 );

    // make the frame to contain the layout
    QFrame * frame1 = new QFrame();
    frame1->setParent( ui->sessionListFrame );
    frame1->setLayout( frame1Layout );
    frame1->setObjectName( name );
    frame1->setStyleSheet( QString(
            "background: rgba( 0,0,0,0 );"
             ));


    // Make the remove button
    QPushButton * pb = new QPushButton(QIcon(":/img2/redX.png"),tr(""), frame1 );
    pb->setObjectName( name );

    pb->setStyleSheet( QString(
           "QPushButton {"
           "max-width: 30px;"
           "min-width: 29px;"
           "text-align: center;"
           "background:    rgba( 0,0,0,0 );"
           "min-height:    2em;"
           "border-width:  0px;"
           "border-radius: 5px;"
           "padding-left:  5px;"
           "padding-right: 5px;"
           "}"

           "QPushButton:hover {"
           "background: rgba( 0,0,0,60);"
           "border-radius: 5px;"
           "border:        0px;"
           "}"

           "QPushButton:pressed {"
           "border-width:  1px;"
           "border-color:  red;"
           "border-style:  solid;"
           "border-radius: 5px;"
           "background: rgba( 255,0,0,30);"
           "}"
           ));

    frame1Layout->addWidget( pb );
    connect( pb, SIGNAL(clicked()), this, SLOT(removeSessionButton_clicked()));

    // Make the session label
    pb = new QPushButton( name, frame1 );
    pb->setStyleSheet( QString(

    "QPushButton {"
    "text-align:    left;"
    "background:    rgba( 0,0,0,0 );"
    "min-height:    2em;"
    "border-width:  0px;"
    "border-radius: 5px;"
    "padding-left:  5px;"
    "padding-right: 5px;"
    "}"

    "QPushButton:hover {"
    "background: rgba(0,0,0,50);"
    "border-radius: 5px;"
    "border:        0px;"
    "}"

    "QPushButton:pressed {"
    "border-width:  1px;"
    "border-color:  red;"
    "border-style:  solid;"
    "border-radius: 5px;"
    "background: rgba( 255,0,0,30);"
    "}"

    ));
    pb->setObjectName( name );

    frame1Layout->addWidget( pb );

    // Connect this button to a handler
    syslog( LOG_INFO, "Connecting minimal view button %s.", name.toAscii().data());
    connect( pb, SIGNAL(clicked()), this, SLOT(sessionButtonClicked()));

    // Put the frame into the vertical layout to make it look like a list
    ui->sessionListFrameLayout->insertWidget(0 , frame1, 0, Qt::AlignLeft);
}


// Handle the button clicked event from the session button
void MinimalViewDialog::sessionButtonClicked()
{
    QPushButton * pb = (QPushButton *) this->sender();
    QString name = pb->objectName();

    pb->setDefault( false );

    syslog( LOG_INFO, "Sending a Session Activation request to the main window for %s", name.toAscii().data());

    // Redirect the event with the name of the sender
    emit minimalViewActivateSession( name );
}


// This is the return call from the main window
void MinimalViewDialog::removeSession( const QString name )
{
    QFrame * session;
    QList< QFrame *>frames;

    syslog( LOG_INFO, "Minimal View Session List frame has %i children", ui->sessionListFrameLayout->children().count());
    syslog( LOG_INFO, "Minimal View received remove session request from main window for: %s", name.toAscii().data());

    session = ui->sessionListFrame->findChild<QFrame *>( name );
    syslog( LOG_INFO, "Minimal View Removing session named '%s'", session->objectName().toAscii().data());
    ui->sessionListFrameLayout->removeWidget( session );
    delete session;
    ui->sessionListScrollArea->update();
}

void MinimalViewDialog::removeSessionButton_clicked()
{
    syslog ( LOG_INFO, "'%s' button was pressed", this->sender()->objectName().toAscii().data());

    // Unset the default thing
    QPushButton * pb = (QPushButton *) this->sender();
    pb->setDefault( false );

    emit removeSessionClicked( this->sender()->objectName());
}


void MinimalViewDialog::setMinimalViewFarmLocationLabel( QString text )
{
    ui->locationLabel->setText( text );
}

void MinimalViewDialog::refreshButtonEnable(bool x)
{
    ui->minimalRefreshButton->setEnabled(x);
}

/////////////////////////////////////////////////////////
//
// Automatic Qt button connections
//
/////////////////////////////////////////////////////////

void MinimalViewDialog::on_minimalExpandButton_clicked()
{
    // close this dialog to allow the main one to proceed again
    setVisible( false );
    parentWidget()->move( x(),y());
    parentWidget()->show();
    parentWidget()->showNormal();
}

void MinimalViewDialog::on_minimalMinimizeButton_clicked()
{
    parentWidget()->showMinimized();
    parentWidget()->setVisible( false );
    setVisible( false );
    showMinimized();
}

void MinimalViewDialog::on_minimalRefreshButton_clicked()
{
    emit minimalRefreshButtonClicked();
}

void MinimalViewDialog::on_minimalCloseButton_clicked()
{

    syslog( LOG_WARNING, "App Portal closed from Main View");
    this->close();
}

void MinimalViewDialog::on_minimalFarmMenuButton_clicked()
{
    emit minimalFarmMenuButtonClicked();

}

void MinimalViewDialog::on_minimalSettingsButton_clicked()
{
    AppSettingsDialog dialog;

    dialog.move( QPoint(( x() + width()/2) - dialog.width()/2,
                        ( y() + height()/2) - dialog.height()/2 ));

    dialog.exec();

    syslog( LOG_NOTICE, "Leaving the Settings action on the minimal view");
}

void MinimalViewDialog::on_minimalHelpButton_clicked()
{
    MainAboutDialog dialog;

    dialog.move( QPoint(( x() + width()/2) - dialog.width()/2,
                        ( y() + height()/2) - dialog.height()/2 ));

    dialog.exec();

    syslog( LOG_INFO, "About dialog completed.");
}

void MinimalViewDialog::on_minimalViewApplicationListView_activated(QModelIndex index)
{
    // activate the item at this index.
    emit minimalViewApplicationListViewActivated( index );
}

void MinimalViewDialog::on_UsbrButton_clicked()
{
    UsbrDialog dialog;
    dialog.move( QPoint(( x() + width()/2) - dialog.width()/2,
                        ( y() + height()/2) - dialog.height()/2 ));
    dialog.exec();
}
