// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include "device.h"
#include "ports.h"
#include <QtDebug>
#include <QPrinterInfo>

DeviceDirector::DeviceDirector():
    mount(0)
{

}

DeviceDirector::~DeviceDirector()
{

}


QStringList DeviceDirector::getMountDrives()
{
    QStringList deviceList;

    // Initialize process
    mount = new QProcess();
    mount->start("mount");

    // If process can`t be started
    if (!mount->waitForStarted())
        return deviceList;

    // Wait for the end of process
    if (!mount->waitForFinished())
        return deviceList;

    QByteArray output = mount->readAllStandardOutput();

    // printf("\nDrives: %s.\n\n", output.data());

    // Kill process
    delete(mount);
    mount = 0;

    QStringList list = QString(output).split("\n");
    QStringList res = list.filter(QRegExp("^/"));

    // printf("\nFound %i possible mounted drives.\n", res.size());

    // Getting device list
    for (int i = 0; i < res.size(); ++i)
    {
        // printf( "   Parsing '%s'.\n", res.at(i).toAscii().data());
        QStringList tmp = QString(res.at(i).toLocal8Bit().constData()).split(" ");
        QString drivePath = tmp.at(2).toLocal8Bit().constData();

        int slashIndex = drivePath.indexOf("/", 1);
        QString driveName = drivePath.mid(1, slashIndex - 1);

        // Hardcoded handler for the root of the file system
        // There should be a window in the future along the lines of
        // conn manager to only share certain paths.
        if ( !drivePath.compare( QString("/"))) driveName = "root";

        //deviceList << driveName + "=" + drivePath;
        //
        // If the the drive name is valid, then add it to the list
        //
        if (!driveName.isEmpty())
        {
            deviceList << driveName + "=" + drivePath;
//             printf( "Adding path:%s as named drive:%s.\n",
//                     drivePath.toAscii().data(),
//                     driveName.toAscii().data());
        }
    }

    return deviceList;
}

QStringList DeviceDirector::getComPorts()
{
    QStringList comPortsList;

    // Getting prefix for com ports
    char *comPrefix = getPrefix("/proc/tty/drivers");

    // If prefix doesn`t exist
    if (comPrefix == 0)
        return comPortsList;

    // Try to get serial ports list
    QStringList ports = findTtyS("/dev", "ttyS");

    // Creating option list
    for (int i = 0; i < ports.count(); i++)
    {
        comPortsList << "COM" + QString::number(i + 1) + "=" + ports.at(i);
    }

    // printf ( "My COM Port list is: %s\n", comPortsList.join( QString("|")).toAscii().data());
    fflush (stdout);
    return comPortsList;
}

QStringList DeviceDirector::getLptPorts()
{
    QStringList lptPortsList;

    // Getting prefix for lpt ports
    char *lptPrefix = getPrefix("/proc/tty/drivers");

    // If prefix doesn`t exist
    if (lptPrefix == 0)
        return lptPortsList;

    // Try to get serial ports list
    QStringList ports = findTtyS("/dev", "lp");

    // Creating option list
    for (int i = 0; i < ports.count(); i++)
        lptPortsList << "LPT" + QString::number(i + 1) + "=" + ports.at(i);

    // printf ( "My LPT Port list is: %s\n", lptPortsList.join( QString("|")).toAscii().data());
    fflush (stdout);
    return lptPortsList;
}
