// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include "verticalMenuItem.h"

VerticalMenuItem::VerticalMenuItem(QAction *action, QWidget *pwgt/*=0*/)
	: QWidget(pwgt)
{
    const int itemHeight = 20;
    const int frstItemOffsetX = 0;
    const int scndItemOffsetX = 24;
    const int itemOffsetY = 1;
    const int iconSize = 16;


	parentAction = action;
	QObject::connect(action, SIGNAL(changed()), this, SLOT(parentActionChanged()));

	imageLabel = new QLabel(this);
    imageLabel->setPixmap(parentAction->icon().pixmap(iconSize, iconSize));
    QRect rect = imageLabel->geometry();
	rect.setX(frstItemOffsetX);
	rect.setY(itemOffsetY);
	imageLabel->setGeometry(rect);
	imageLabel->adjustSize();

	textLabel = new QLabel(this);
	textLabel->setText(parentAction->text().replace("&", ""));
	rect = textLabel->geometry();
	rect.setX(scndItemOffsetX);
	rect.setY(itemOffsetY);
	textLabel->setGeometry(rect);
	textLabel->adjustSize();

    this->setEnabled(parentAction->isEnabled());
	this->setVisible(parentAction->isVisible());
    this->setMaximumHeight(itemHeight);
    this->setMinimumHeight(itemHeight);
    this->setStyleSheet(QString("border: 0px; margin: 0px; padding: 0px;"));
}

/*virtual*/ void VerticalMenuItem::mouseReleaseEvent(QMouseEvent *)
{
	if (this->isEnabled())
		parentAction->activate(QAction::Trigger);
}

/*virtual*/ void VerticalMenuItem::parentActionChanged()
{
	this->setEnabled(parentAction->isEnabled());
	this->setVisible(parentAction->isVisible());
}

/*virtual*/ void VerticalMenuItem::enterEvent(QEvent *)
{
	if (this->isEnabled())
	{
		QFont font = textLabel->font();
		font.setUnderline(true);
		textLabel->setFont(font);

		parentAction->activate(QAction::Hover);
	}
}
/*virtual*/ void VerticalMenuItem::leaveEvent(QEvent *)
{
	if (this->isEnabled())
	{
		QFont font = textLabel->font();
		font.setUnderline(false);
		textLabel->setFont(font);
	}
}
