// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

/*
 *
 * Copyright (C) 2012, Quest Software
 *
 */

#include <string>
#include <memory>

#include "../common/common.h"
#include "FlashPluginUtils.h"

#include <pwd.h>
#include <limits.h>


//  Below is a copypaste from 
//  fmmr/addin/src/TSFlashClient/FlashFactory.cpp
char const *getUserFlashDir()
{
    char const *home;
    static char path[PATH_MAX];
    struct passwd *pwent = ::getpwuid(::geteuid());

    if (pwent)
        home = pwent->pw_dir;
    else
        home = ::getenv("HOME");

    if (home)
    {
        ::snprintf(path, PATH_MAX, "%s/.mozilla/plugins", home);

        return path;
    }

    return 0;
}

char const **getFlashDirs()
{
    static char const *default_dirs[] =
    {
        "/usr/lib/flash-plugin",
        "/usr/lib/adobe-flashplugin",
        "/usr/lib32/adobe-flashplugin",
        "/usr/lib64/adobe-flashplugin",
        "/usr/lib/flashplugin-installer",
        "/usr/lib32/flashplugin-installer",
        "/usr/lib64/flashplugin-installer",
        "/usr/lib/flashplugin-nonfree",
        "/usr/lib32/flashplugin-nonfree",
        "/usr/lib64/flashplugin-nonfree",
        "/usr/lib/mozilla/plugins",
        "/usr/lib32/mozilla/plugins",
        "/usr/lib64/mozilla/plugins",
        "/usr/lib/firefox/plugins",
        "/usr/lib32/firefox/plugins",
        "/usr/lib64/firefox/plugins",
        "/usr/lib/iceweasel/plugins",
        "/usr/lib32/iceweasel/plugins",
        "/usr/lib64/iceweasel/plugins",
        "/usr/lib/seamonkey/plugins",
        "/usr/lib32/seamonkey/plugins",
        "/usr/lib64/seamonkey/plugins",
        "/usr/lib/nsbrowser/plugins",
        "/usr/lib32/nsbrowser/plugins",
        "/usr/lib64/nsbrowser/plugins",
        "/usr/lib/browser-plugins",
        "/usr/lib32/browser-plugins",
        "/usr/lib64/browser-plugins",
        "/usr/lib/mozilla-firefox/plugins",
        "/usr/lib32/mozilla-firefox/plugins",
        "/usr/lib64/mozilla-firefox/plugins",
        "/opt/netscape/plugins",
        "/usr/local/netscape/plugins",
        "/opt/mozilla/plugins",
        "/opt/mozilla/lib/plugins",
        "/opt/MozillaFirefox/lib/plugins",
        "/opt/MozillaFirefox/lib32/plugins",
        "/opt/MozillaFirefox/lib64/plugins",
        "/lib"
    };

    int const n_default_dirs = sizeof(default_dirs) / sizeof(size_t);
    char const **dirs = new char const *[(n_default_dirs + 2) * sizeof(size_t)];

    int i, j;

    for (i = 0, j = 0; i < n_default_dirs; ++i)
    {
        char const *dir = default_dirs[i];

        if (dir && 0 == ::access(dir, F_OK))
            dirs[j++] = dir;
    }

    dirs[j++] = getUserFlashDir();
    dirs[j] = 0;

    return dirs;
}

bool isFlashPluginInstalled()
{
    std::auto_ptr<char const *>dirs(getFlashDirs());

    for (unsigned i = 0; dirs.get()[i]; ++i)
    {
        std::string name = dirs.get()[i] + std::string("/"ADOBE_FLASH_PLUGIN_FILENAME);

        if (0 == ::access(name.c_str(), R_OK))
        {
            return true;
        }
    }

    return false;
}

