// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

/*
 *
 * Copyright (C) 2010, Quest Software
 *
 */
#include <ifaddrs.h>
#include <arpa/inet.h>
#include <limits.h>
#include <netdb.h>
#include <unistd.h>

#ifdef __cplusplus
extern "C" {
#endif
#include "../common/common.h"

/////////////////////////////////////////////////
//
// Application global function definitions go HERE
//
/////////////////////////////////////////////////

int linktest( char * dummy )
{
    syslog( LOG_NOTICE, "Common utils link test" );
    return 0; 
}

// This code is used both in Qrdesktop and pncm.
void getHostName(char *hostname)
{
    int result = gethostname(hostname, HOST_NAME_MAX);
    if (result != 0)
    {
        syslog(LOG_CRIT, "Critical: could not determine hostname!");
        exit(1);
    }

    // Getting FQDN in he->h_name by host name
    struct hostent* he = gethostbyname(hostname);
    if (he != NULL)
    {
        strcpy(hostname, he->h_name);
    }
}

// Try to get the IP Address with the default gateway,
// or the first non-loopback on if no default path
// exists
// This code is used both in Qrdesktop and pncm.
int getIpAddress(char *ipaddress)
{
    struct ifaddrs * ifAddrStruct=NULL;
    struct ifaddrs * ifa=NULL;
    void * tmpAddrPtr=NULL;
    char ipAddressString[80];

    getifaddrs(&ifAddrStruct);

    for (ifa = ifAddrStruct; ifa != NULL; ifa = ifa->ifa_next) {

        // ifa ->ifa_addr returns NULL for "pppX" interface (VPN connection)
        if (ifa->ifa_addr == NULL)
        {
            continue;
        }

        if (ifa->ifa_addr->sa_family==AF_INET) { // check it is IP4
            // is a valid IP4 Address
            tmpAddrPtr=&((struct sockaddr_in *)ifa->ifa_addr)->sin_addr;
            char addressBuffer[INET_ADDRSTRLEN];
            inet_ntop(AF_INET, tmpAddrPtr, addressBuffer, INET_ADDRSTRLEN);
            syslog(LOG_INFO, "%s IP Address %s\n", ifa->ifa_name, addressBuffer);

            // Handle the case for IPv4 only
            strcpy( ipaddress, addressBuffer );

            // If the first octet is NOT a loopback or invalid octet then save this and return it.
            int firstOctet = strtol (strtok( addressBuffer, "."), NULL, 10);
            if (( firstOctet != 0) && (firstOctet != 127))
            {
                syslog ( LOG_INFO, "Choosing %s", ipaddress );
                if (ifAddrStruct!=NULL) freeifaddrs(ifAddrStruct);
                return (0);
            }

        } else if (ifa->ifa_addr->sa_family==AF_INET6) { // check it is IP6
            // is a valid IP6 Address
            tmpAddrPtr=&((struct sockaddr_in *)ifa->ifa_addr)->sin_addr;
            char addressBuffer[INET6_ADDRSTRLEN];
            inet_ntop(AF_INET6, tmpAddrPtr, addressBuffer, INET6_ADDRSTRLEN);
            syslog(LOG_INFO, "%s IP Address %s\n", ifa->ifa_name, addressBuffer);
        }
    }
    if (ifAddrStruct!=NULL) freeifaddrs(ifAddrStruct);
    return (-1);
}

#ifdef __cplusplus
}
#endif
