// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

/*
 *
 * Copyright (C) 2010, Quest Software
 *
 */

#pragma once


/////////////////////////////////////////////////
//
// Application wide system #includes go here
//
/////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <string.h>
#include "version.h"

/////////////////////////////////////////////////
//
// Application wide app includes go here
//
/////////////////////////////////////////////////



/////////////////////////////////////////////////
//
// Application wide values go here
//
/////////////////////////////////////////////////



/////////////////////////////////////////////////
//
// Application wide macros go here
//
/////////////////////////////////////////////////

// Build control flags

#ifdef _OEM_BUILD
    #define APP_BUILT_TYPE           "OEM"
#elif DEBUG
    #define APP_BUILT_TYPE           "Dev"
#else
    #define APP_BUILT_TYPE           "FINAL"
#endif // ifdef _OEM_BUILD

// Ticket location
#define Q_SECURE_TICKET_FOLDER_SHELL_COMMAND_TEMPLATE "%s /tmp/%s/vWorkspace/ticket"
#define Q_SECURE_TICKET_FILE_TEMPLATE                 "/tmp/%s/vWorkspace/ticket/qrdesktop_tix.bin"

// Pit file from web launcher location
#define Q_PIT_FILE_FOLDER_SHELL_COMMAND_TEMPLATE      "%s/tmp/%s/vWorkspace"

// Application config file name and key names for values search in it
#define APPLICATION_CONFIG_FILE_NAME                  "/etc/appPortalSettings.ini"
#define FARM_OVERRIDE_VALUES                          "Farm override values"
#define FARM_DEFAULT_VALUES                           "Farm default values"
#define GUI_TRANSLATION_FILE_PATH                     "/usr/local/share/qrdesktop"

#define PNINFO_CONNECTOR_TYPE "Linux"
#define PNINFO_DEVICE_TYPE "Linux Computer"

/////////////////////////////////////////////////
//
// A word about build descriptions:
//
// It is intended that there be 3 kinds of builds,
//
//   * dev - for development use only
//
//   * standard - non debug builds for version testing
//
//   * final - builds to be submitted to QA/RTM
//
//  DEV builds are for normal everyday coding and testing.
//  STANDARD builds are non-debug builds that are still not ready for
//      submitting to QA to RTM.  This may be necessary for testing before
//      the FINAL builds are done.
//  FINAL builds are checkpoint builds that are meant to be submitted
//      to QA or RTM.  There can only be ONE (1) FINAL build per build number.
//
/////////////////////////////////////////////////


#ifndef DEBUG
//#define BUILD_DESCRIPTION_STRING "Normal (non-debug) build"
#define BUILD_DESCRIPTION_STRING "Final (non-debug) build"
#else 
#define BUILD_DESCRIPTION_STRING "WARNING: DEBUG BUILD (except for qrdesktop)"
#endif

// Use a macro for this to keep it simple
#define LOG_BANNER( x )                  syslog( LOG_NOTICE, "+" );\
        syslog( LOG_NOTICE, "Starting %s.  Build date: %s Build time: %s.", x, __DATE__, __TIME__ );\
        syslog( LOG_NOTICE, "App version: %s %s.", APP_VERSION_STRING, APP_BUILD_STRING);\
        syslog( LOG_NOTICE, "Description: %s", BUILD_DESCRIPTION_STRING );\
        syslog( LOG_NOTICE, "+" );



//////////////////////////////////////////////////
//
// Thumbnail updater struct definitions are here
//
//////////////////////////////////////////////////

#pragma pack(push, 1)


#define AU_NETWORK_PORT                 (20152)

#define AU_ACTION_UPDATE_THUMBNAIL      (0)
#define AU_ACTION_REMOVE_SESSION        (1)

//
//  NOTE: Please restrict variable types and array sized to end on 
//        a 4 byte boundary.  This will prevent future alignment problems
//        and we can certainly spare the few extra bytes.
//
struct pixel_update_header
{
        int action;        // opcode
        int width;         // pixels
        int height;        // pixels
        int window_id;     // ID of the window as known to the window manager
        int seamless;      // 0 = Not seamless; 1 = Provision style; 2 = MS Style

        char session_name[256];
        
        // Space for more later
};

#pragma pack( pop )




/////////////////////////////////////////////////
//
// Application wide global function decls go here
//
/////////////////////////////////////////////////
#ifdef __cplusplus
extern "C" {
#endif

int linktest( char * );
int getIpAddress( char *ipaddress );
void getHostName(char *hostname);

#ifdef __cplusplus
}
#endif

