#!/bin/bash  

PROJECTDIR=$PWD
BUILDCMD=$1
DEVICE="generic"
CUSTOMER=
CLEAN=
PROCS=
QT_SOURCE="qt-everywhere-opensource-src-4.8.1"

APP_VERSION_STRING="7.6"
APP_VERSION_FULL_STRING="7.6.0.10"
APP_BUILD_STRING="Build 10"
APP_COPYRIGHT_STRING="Copyright 2012 Quest Software, Inc."

cat <<VERSIONSET > $PROJECTDIR/common/version.h
#pragma once
#define APP_VERSION_STRING           "$APP_VERSION_STRING"
#define APP_VERSION_FULL_STRING      "$APP_VERSION_FULL_STRING"
#define APP_BUILD_STRING             "$APP_BUILD_STRING"
#define APP_COPYRIGHT_STRING         "$APP_COPYRIGHT_STRING"
static const char STATIC_COPYRIGHT_STRING[] =  "Version $APP_VERSION_FULL_STRING $APP_COPYRIGHT_STRING";
VERSIONSET
        
for arg in "$@"
    do
    case $arg in
        help)
            echo " all-libs all-libs-forked all extensions commonlib fmmr mmrclient mmrproxy proxying[_config] usbrdr vworkspace pntsc pncm hprgs qrdesktop[_config] connectionbar ice qt libao libmpg123 libboost libssl libpopt"
            echo
            echo "for brand new system:"
            echo "bin/build.sh prerequisites"
            echo "bin/build.sh all-libs generic"
            echo "bin/build.sh all generic "
            echo "bin/build.sh output generic"
            
            exit 1
            ;;
            
        clean)
            CLEAN=" clean"
            ;;

        fast)
            #make processes = #cpu + 1
            #this example is 2 cpu
            PROCS=-j3
            ;;

        generic)
            DEVICE="generic"
            ;;
            
    esac
done



case "$BUILDCMD" in
    prerequisites)
        PACKAGE_INSTALL_CMD='sudo apt-get -y install '
        sudo apt-get -y update
        $PACKAGE_INSTALL_CMD build-essential
        #$PACKAGE_INSTALL packaging-dev
        #$PACKAGE_INSTALL_CMD default-jre
        #$PACKAGE_INSTALL_CMD p7zip-full

        $PACKAGE_INSTALL_CMD pkg-config
        $PACKAGE_INSTALL_CMD autoconf
        $PACKAGE_INSTALL_CMD automake
        $PACKAGE_INSTALL_CMD libtool
        $PACKAGE_INSTALL_CMD libpng12-dev
        $PACKAGE_INSTALL_CMD libmpg123-dev
        $PACKAGE_INSTALL_CMD libjpeg62-dev
        $PACKAGE_INSTALL_CMD libssl-dev
        $PACKAGE_INSTALL_CMD libXext-dev
        $PACKAGE_INSTALL_CMD libasound2-dev
        $PACKAGE_INSTALL_CMD libx11-dev
        $PACKAGE_INSTALL_CMD libxt-dev
        $PACKAGE_INSTALL_CMD libxcursor-dev
        $PACKAGE_INSTALL_CMD xdg-utils
        $PACKAGE_INSTALL_CMD libpcsclite1
        $PACKAGE_INSTALL_CMD libpcsclite-dev
        $PACKAGE_INSTALL_CMD libxerces-c-dev
        $PACKAGE_INSTALL_CMD libspeex-dev
        $PACKAGE_INSTALL_CMD libgstreamer0.10-dev
        $PACKAGE_INSTALL_CMD libgstreamer-plugins-base0.10-dev
        $PACKAGE_INSTALL_CMD gstreamer0.10-tools
        $PACKAGE_INSTALL_CMD gstreamer0.10-plugins-base
        $PACKAGE_INSTALL_CMD gstreamer0.10-plugins-good
        $PACKAGE_INSTALL_CMD gstreamer0.10-plugins-bad
        $PACKAGE_INSTALL_CMD gstreamer0.10-plugins-ugly
        $PACKAGE_INSTALL_CMD gstreamer0.10-ffmpeg
        $PACKAGE_INSTALL_CMD gstreamer0.10-pitfdll
        $PACKAGE_INSTALL_CMD libxinerama-dev
        $PACKAGE_INSTALL_CMD gdebi
        $PACKAGE_INSTALL_CMD wmctrl
        $PACKAGE_INSTALL_CMD libsocksd0-dev
        $PACKAGE_INSTALL_CMD ffmpeg
        $PACKAGE_INSTALL_CMD libgtk2.0-dev
        $PACKAGE_INSTALL_CMD libnspr4-dev
        $PACKAGE_INSTALL_CMD libnss3-dev
        $PACKAGE_INSTALL_CMD libxv-dev
        $PACKAGE_INSTALL_CMD libcups2-dev        
        $PACKAGE_INSTALL_CMD uuid-dev
        $PACKAGE_INSTALL_CMD libuuid1
        $PACKAGE_INSTALL_CMD libmcpp0
        $PACKAGE_INSTALL_CMD libmcpp-dev
        $PACKAGE_INSTALL_CMD ant
        $PACKAGE_INSTALL_CMD libbz2-dev
        $PACKAGE_INSTALL_CMD libdb4.6-dev
        $PACKAGE_INSTALL_CMD libdb4.6++-dev
        $PACKAGE_INSTALL_CMD libcups2-dev
        $PACKAGE_INSTALL_CMD  python-dev
        $PACKAGE_INSTALL_CMD bjam
        $PACKAGE_INSTALL_CMD rpm
        #$PACKAGE_INSTALL_CMD build-dep
        $PACKAGE_INSTALL_CMD debian-installer
        ;;



    qt-setup)
        mkdir -p $PROJECTDIR/qt
        mkdir -p $PROJECTDIR/qt/$DEVICE/shadow-shared
        mkdir -p $PROJECTDIR/qt/$DEVICE/shadow-static
        mkdir -p $PROJECTDIR/qt/$DEVICE/output-static
        mkdir -p $PROJECTDIR/qt/$DEVICE/output-shared


        cd $PROJECTDIR/qt

        if [ ! -f "$PROJECTDIR/qt/$QT_SOURCE.tar.gz" ]
        then
            wget ftp://ftp.qt.nokia.com/qt/source/$QT_SOURCE.tar.gz
            tar -zxvf $QT_TARBALL
        fi

        if [ ! -f "$PROJECTDIR/qt/$QT_SOURCE/configure" ]
        then
           tar -zxvf $QT_SOURCE.tar.gz
        fi
        ;;


    qt-static)
        QTDIR=$PROJECTDIR/qt/$DEVICE/output-static
        PATH=$PROJECTDIR/qt/$DEVICE/output-static/bin:$PATH
        cd $PROJECTDIR/qt/$DEVICE/shadow-static

        if test -z $CLEAN
        then
           make $PROCS
           make $PROCS install
        else
           make confclean
           $PROJECTDIR/qt/$QT_SOURCE/configure -opensource -prefix $PROJECTDIR/qt/$DEVICE/output-static -static -nomake docs -nomake examples -nomake demos -release -fast -no-qt3support -no-webkit -silent -continue -plugin-sql-sqlite -plugin-sql-sqlite2 -qt-zlib -qt-libtiff -qt-libpng -qt-libmng -qt-libjpeg -confirm-license
           make $PROCS
           make $PROCS install
        fi
        ;;


    qt-shared)
        QTDIR=$PROJECTDIR/qt/$DEVICE/output-shared
        PATH=$PROJECTDIR/qt/$DEVICE/output-shared/bin:$PATH
        cd $PROJECTDIR/qt/$DEVICE/shadow-shared
        if test -z $CLEAN
        then
           make $PROCS
           make $PROCS install
        else
           make confclean
           $PROJECTDIR/qt/$QT_SOURCE/configure -opensource -prefix $PROJECTDIR/qt/$DEVICE/output-shared -shared -nomake docs -nomake examples -nomake demos -release -fast -no-qt3support -no-webkit -silent -continue -plugin-sql-sqlite -plugin-sql-sqlite2 -qt-zlib -qt-libtiff -qt-libpng -qt-libmng -qt-libjpeg -confirm-license
           make $PROCS
           make $PROCS install
        fi
        ;;  


    qt)
        ./$0 qt-setup   $CLEAN $DEVICE $CUSTOMER
        ./$0 qt-static  $CLEAN $DEVICE $CUSTOMER
        ./$0 qt-shared  $CLEAN $DEVICE $CUSTOMER
        ;;       


    ice)
        cd $PROJECTDIR/ice
        mkdir output
        tar -zxvf Ice-3.4.2.tgz
        #cp Buffer.h Ice-3.4.2/cpp/include/Ice/Buffer.h
        cd Ice-3.4.2
        make $PROCS cpp
        cd cpp
        make $PROCS prefix="$PROJECTDIR/ice/output" install
        ;;        

    libao)
        cd $PROJECTDIR/qrdesktop/libao
        tar -zxvf libao-1.1.0.tar.gz
        cd  $PROJECTDIR/qrdesktop/libao/libao-1.1.0
        /bin/sh ./configure  --prefix=$PROJECTDIR/qrdesktop/libao/output --enable-static
        make $PROCS
        make $PROCS install
        ;;

    libpopt)
        cd $PROJECTDIR/qrdesktop/libpopt
        tar -zxvf popt-1.16.tar.gz 
        cd  $PROJECTDIR/qrdesktop/libpopt/popt-1.16
        /bin/sh ./configure  --prefix=$PROJECTDIR/qrdesktop/libpopt/output --enable-static
        make $PROCS
        make $PROCS install
        ;;

    libmpg123)
        cd $PROJECTDIR/qrdesktop/libmpg123
        tar -zxvf mpg123-1.14.3.tgz
        cd $PROJECTDIR/qrdesktop/libmpg123/mpg123-1.14.3
        /bin/sh ./configure  --prefix=$PROJECTDIR/qrdesktop/libmpg123/output --enable-static
        make $PROCS
        make $PROCS install
        ;;

    libssl)
        cd $PROJECTDIR/qrdesktop/libssl
        tar -zxvf openssl-0.9.8x.tar.gz
        cd $PROJECTDIR/qrdesktop/libssl/openssl-0.9.8x
        /bin/sh ./config  no-shared --prefix=$PROJECTDIR/qrdesktop/libssl/output
        make $PROCS
        make $PROCS install
        ;;

    libboost)
        cd $PROJECTDIR/qrdesktop/libboost
        tar -zxvf boost_1_50_0.tar.gz
        cd $PROJECTDIR/qrdesktop/libboost/boost_1_50_0
        /bin/sh ./bootstrap.sh --prefix=$PROJECTDIR/qrdesktop/libboost/output
        ./bjam
        ./bjam install
        ;;

    proxying_config)
        if test -z $CLEAN
        then
           cd $PROJECTDIR/qrdesktop/dante_proxy
           rm -rf $PROJECTDIR/qrdesktop/dante_proxy/dante-1.3.2
           tar zxvf dante-1.3.2.tar.gz
           cd dante-1.3.2
           /bin/sh configure --enable-static --disable-shared --enable-debug --without-glibc-secure  --without-gssapi --without-krb5 --without-sasl --without-ldap
        fi
        ;;

    proxying)
        cd $PROJECTDIR/qrdesktop/dante_proxy/dante-1.3.2

        if test -z $CLEAN
        then
            make $PROCS  DEVICE=$DEVICE CUSTOMER=$CUSTOMER
            cp $PROJECTDIR/qrdesktop/dante_proxy/dante-1.3.2/lib/.libs/libsocks.a   $PROJECTDIR/qrdesktop/dante_proxy/libsocksd.a
        else
            rm qrdesktop/dante_proxy/dante-1.3.2/lib/.libs/libsocks.a
            rm $PROJECTDIR/qrdesktop/dante_proxy/libsocksd.a
            make clean;
        fi
        ;;

    all-libs-forked)
        set -m
        ./$0 libpopt                  $DEVICE $CUSTOMER &
        ./$0 libao                    $DEVICE $CUSTOMER &
        ./$0 libmpg123                $DEVICE $CUSTOMER &
        ./$0 libssl                   $DEVICE $CUSTOMER &
        ./$0 libboost                 $DEVICE $CUSTOMER & 
        ./$0 qt                clean  $DEVICE $CUSTOMER &
        while [ 1 ]; do fg 2> /dev/null; [ $? == 1 ] && break; done

        ./$0 ice                      $DEVICE $CUSTOMER
        ./$0 proxying_config          $DEVICE $CUSTOMER 
        ./$0 proxying                 $DEVICE $CUSTOMER
        ./$0 commonlib                $DEVICE $CUSTOMER
        ;;

    all-libs)
        set -m
        ./$0 libpopt                  $DEVICE $CUSTOMER 
        ./$0 libao                    $DEVICE $CUSTOMER 
        ./$0 libmpg123                $DEVICE $CUSTOMER 
        ./$0 libssl                   $DEVICE $CUSTOMER 
        ./$0 libboost                 $DEVICE $CUSTOMER  
        ./$0 qt                clean  $DEVICE $CUSTOMER 
        ./$0 ice                      $DEVICE $CUSTOMER
        ./$0 proxying_config          $DEVICE $CUSTOMER 
        ./$0 proxying                 $DEVICE $CUSTOMER
        ./$0 commonlib                $DEVICE $CUSTOMER
        ;;

    all)
        ./$0 commonlib         $CLEAN $DEVICE $CUSTOMER
        ./$0 extensions        $CLEAN $DEVICE $CUSTOMER
        ./$0 vworkspace        $CLEAN $DEVICE $CUSTOMER 
        ./$0 connectionbar     $CLEAN $DEVICE $CUSTOMER
        ./$0 qrdesktop_config  $CLEAN $DEVICE $CUSTOMER  
        ./$0 qrdesktop         $CLEAN $DEVICE $CUSTOMER
        ./$0 pntsc             $CLEAN $DEVICE $CUSTOMER  
        ./$0 pncm              $CLEAN $DEVICE $CUSTOMER
        ;;


    all-forked)
        ./$0 commonlib         $CLEAN $DEVICE $CUSTOMER &
        ./$0 pntsc             $CLEAN $DEVICE $CUSTOMER &
        ./$0 pncm              $CLEAN $DEVICE $CUSTOMER &
        ./$0 qrdesktop_config  $CLEAN $DEVICE $CUSTOMER &
        while [ 1 ]; do fg 2> /dev/null; [ $? == 1 ] && break; done
        ./$0 vworkspace        $CLEAN $DEVICE $CUSTOMER &
        ./$0 qrdesktop         $CLEAN $DEVICE $CUSTOMER &
        while [ 1 ]; do fg 2> /dev/null; [ $? == 1 ] && break; done
        ;;

    extensions)
        ./$0 usbrdr            $CLEAN $DEVICE $CUSTOMER 
        ;;

    commonlib)
        cd $PROJECTDIR/lib
        make $PROCS $CLEAN DEVICE=$DEVICE CUSTOMER=$CUSTOMER
        ;;

    vworkspace)
        cd $PROJECTDIR/appportal2
        QTDIR=$PROJECTDIR/qt/$DEVICE/output-static
        PATH=$PROJECTDIR/qt/$DEVICE/output-static/bin:$PATH
        qmake 
        make $PROCS $CLEAN DEVICE=$DEVICE CUSTOMER=$CUSTOMER
        ;;
      
    connectionbar)
        QTDIR=$PROJECTDIR/qt/$DEVICE/output-static
        PATH=$PROJECTDIR/qt/$DEVICE/output-static/bin:$PATH
        cd $PROJECTDIR/qrdesktop/connectionbar
        qmake
        make $PROCS $CLEAN DEVICE=$DEVICE CUSTOMER=$CUSTOMER
        ;;
   
    pntsc)
        cd $PROJECTDIR/qrdesktop/pntsc
        
        make $PROCS $CLEAN DEVICE=$DEVICE CUSTOMER=$CUSTOMER
        ;;

    pncm)
        #depends on proxying libsocksd.a
        cd $PROJECTDIR/qrdesktop/server
        make $PROCS $CLEAN DEVICE=$DEVICE CUSTOMER=$CUSTOMER
        ;;
       
    qrdesktop_config)
        if test -z $CLEAN
        then
          cd $PROJECTDIR/qrdesktop/rdesktop
          autoconf
          cp configure.good configure
          LIBAO_LIBS=$PROJECTDIR/qrdesktop/libao/output/lib/libao.a ./configure --enable-static --enable-smartcard --with-mm-support --with-openssl=$PROJECTDIR/qrdesktop/libssl/output --enable-static-openssl 
        fi
        ;;

    qrdesktop)
        #depends on proxying libsocksd.a
        cd $PROJECTDIR/qrdesktop/rdesktop
        make $PROCS $CLEAN DEVICE=$DEVICE CUSTOMER=$CUSTOMER
        ;;

    output)
        if test -z $DEVICE
        then
           DEVICE=generic
        fi           
       
        cd $PROJECTDIR
        mkdir -p output/
        rm   -rf output/$DEVICE
        mkdir -p output/$DEVICE
        mkdir -p output/$DEVICE/etc
        mkdir -p output/$DEVICE/etc/mmr
        mkdir -p output/$DEVICE/etc/xdg/menus
        mkdir -p output/$DEVICE/usr/local/bin
        mkdir -p output/$DEVICE/usr/local/src/usb-redirector
        mkdir -p output/$DEVICE/usr/local/src/usb-redirector/files
        mkdir -p output/$DEVICE/usr/local/src/usb-redirector/files/modules/src

        mkdir -p output/$DEVICE/usr/local/man
        mkdir -p output/$DEVICE/usr/local/man/man1
        mkdir -p output/$DEVICE/usr/local/share
        mkdir -p output/$DEVICE/usr/local/share/applications
        mkdir -p output/$DEVICE/usr/local/share/pixmaps
        mkdir -p output/$DEVICE/usr/local/share/qrdesktop
        mkdir -p output/$DEVICE/usr/local/share/doc/vworkspace

        cp -f appportal2/vworkspace                                    output/$DEVICE/usr/local/bin
        cp -f qrdesktop/connectionbar/connectionbar                    output/$DEVICE/usr/local/bin
        cp -f qrdesktop/pntsc/pntsc                                    output/$DEVICE/usr/local/bin
        cp -f qrdesktop/server/pncm                                    output/$DEVICE/usr/local/bin
        cp -f qrdesktop/rdesktop/qrdesktop                             output/$DEVICE/usr/local/bin

        cp -f qrdesktop/pntsc/launcher                                 output/$DEVICE/usr/local/bin
        chmod 755 output/$DEVICE/usr/local/bin/launcher
        strip     output/$DEVICE/usr/local/bin/launcher              


        cp -rf qrdesktop/rdesktop/keymaps                              output/$DEVICE/usr/local/share/qrdesktop
        cp -rf doc/html                                                output/$DEVICE/usr/local/share/doc/vworkspace
        cp -f  doc/README*                                             output/$DEVICE/usr/local/share/doc/vworkspace
        cp -f  doc/copyright                                           output/$DEVICE/usr/local/share/doc/vworkspace
        cp -f  appportal2/AppPortal_en.qm                              output/$DEVICE/usr/local/share/qrdesktop
        cp -f  installer/deb/qrdesktop-session.xml                     output/$DEVICE/usr/local/share/qrdesktop
        cp -f  installer/deb/vworkspace.desktop                        output/$DEVICE/usr/local/share/applications
        cp -f  installer/deb/qrdesktop-launcher.desktop                output/$DEVICE/usr/local/share/applications
        cp -f  appportal2/img2/Application_icon_32x32.png              output/$DEVICE/usr/local/share/pixmaps/vworkspace.png
        cp -f  installer/deb/menu                                      output/$DEVICE/etc/xdg/menus/vworkspace.menu
        cp -f  qrdesktop/rdesktop/doc/rdesktop.1                       output/$DEVICE/usr/local/man/man1

        cp -f appportal2/common/appPortalSettings.ini                  output/$DEVICE/etc
        
        cp -f extensions/mmr/configs/config                            output/$DEVICE/etc/mmr/config
 
        chmod 644 output/$DEVICE/usr/local/man/man1/rdesktop.1
        chmod 644 output/$DEVICE/etc/mmr/config
        chmod 755 output/$DEVICE/usr/local/bin/pntsc
        chmod 755 output/$DEVICE/usr/local/bin/pncm
        chmod 755 output/$DEVICE/usr/local/bin/qrdesktop
        chmod 755 output/$DEVICE/usr/local/bin/vworkspace
        chmod 755 output/$DEVICE/usr/local/bin/connectionbar


        strip output/$DEVICE/usr/local/bin/pntsc
        strip output/$DEVICE/usr/local/bin/pncm
        strip output/$DEVICE/usr/local/bin/vworkspace
        strip output/$DEVICE/usr/local/bin/connectionbar
        strip output/$DEVICE/usr/local/bin/qrdesktop

	#  delete RPATH defines from binaries
	if [ -x /usr/bin/chrpath ]; then
            /usr/bin/chrpath --delete output/$DEVICE/usr/local/bin/pntsc
            /usr/bin/chrpath --delete output/$DEVICE/usr/local/bin/pncm
            /usr/bin/chrpath --delete output/$DEVICE/usr/local/bin/vworkspace
            /usr/bin/chrpath --delete output/$DEVICE/usr/local/bin/connectionbar
            /usr/bin/chrpath --delete output/$DEVICE/usr/local/bin/qrdesktop
	fi

        #tar package build
        cd $PROJECTDIR/output
        tar -cz $DEVICE > vworkspace-$DEVICE-$APP_VERSION_FULL_STRING.tgz
        ;;
        
esac

exit 1
