// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************


#include "ui_connectionbarwindow.h"
#include "connectionbarwindow.h"

ConnectionBarWindow::ConnectionBarWindow(QWidget* parent) :
    QMainWindow(parent),
    m_sessionName(""),
    m_mainLabelHeight(0),
    m_qrdesktopPID(0),
    ui(new Ui::ConnectionBarWindow)
{
    ui->setupUi(this);
}
ConnectionBarWindow::ConnectionBarWindow(std::string sessionName, pid_t qrdesktopPID, QWidget* parent) :
    QMainWindow(parent),
    m_sessionName(sessionName),
    m_mainLabelHeight(0),
    m_qrdesktopPID(qrdesktopPID),
    ui(new Ui::ConnectionBarWindow)
{
    ui->setupUi(this);
    m_mainLabelHeight = ui->mainLabel->height();
    ui->sessionNameLabel->setText(sessionName.c_str());
}

ConnectionBarWindow::~ConnectionBarWindow()
{
}

void ConnectionBarWindow::on_minimizeButton_clicked()
{
    // SIGUSR2 minimize qrdesktop window signal
    int res = kill(m_qrdesktopPID, SIGUSR2);
    if(res < 0)
    {
        syslog(LOG_WARNING, "Failed to send SIGUSR2 signal to qrdesktop. Error: %i", errno);
        QCoreApplication::exit(1);
    }

    QCoreApplication::exit(0);
}

void ConnectionBarWindow::on_closeButton_clicked()
{
    // SIGUSR1 close qrdesktop window signal
    int res = kill(m_qrdesktopPID, SIGUSR1);
    if(res < 0)
    {
        syslog(LOG_WARNING, "Failed to send SIGUSR1 signal to qrdesktop. Error: %i", errno);
        QCoreApplication::exit(1);
    }

    QCoreApplication::exit(0);
}

void ConnectionBarWindow::on_showConnectionBarButton_clicked()
{
    if(this->y() == 0)
    {
        QPixmap iconPixmap(":/images/images/Toolbar_Arrow_Show.png");
        QIcon icon(iconPixmap);
        this->ui->showConnectionBarButton->setIcon(icon);

        this->move(this->x(), -m_mainLabelHeight);
    }
    else
    {
        QPixmap iconPixmap(":/images/images/Toolbar_Arrow_Hide.png");
        QIcon icon(iconPixmap);
        this->ui->showConnectionBarButton->setIcon(icon);

        this->move(this->x(), 0);
    }
}
