// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include <assert.h>
#include <pwd.h>
#include <unistd.h>
#include "../../common/common.h"

//
// This file is only used to hand off the incoming pit file
// from the wbe browser to the pntsc executable because
// browsers cannot be assumed to allow command line customizations
// and as such the handler application can only take the single
// command line argument as the name of the file.
//
// So 'launcher <file>' turns into 'pntsc -f /tmp/<user_name>/vWorkspace/<file>'
//
// So this make things just a bit cleaner is all.
//

int main ( int argc, char ** argv)
{
    char buffer[1024];
    char * marker;

    if ( argc < 2 ) exit( -1 );

#ifdef DEBUG
    printf( "Incoming file is %s\n", argv[1] );
    syslog( LOG_NOTICE, "Launcher handling: '%s'", argv[1] );
#endif

    // Strip the path to leave  only the file name
    marker = strrchr( argv[1], '/' );

    openlog( "launcher", LOG_PID, LOG_USER );
    char userSysName[128];
    struct passwd *pw;
    pw = getpwuid( getuid() );
    if ( pw == NULL || pw->pw_name == NULL )
    {
        syslog( LOG_ERR, "Unable to get system user name!" );
        assert( pw != NULL && pw->pw_name != NULL );
    }
    else
    {
        strcpy( userSysName, pw->pw_name );
    }
    sprintf( buffer, Q_PIT_FILE_FOLDER_SHELL_COMMAND_TEMPLATE, "mkdir -p ", userSysName );
    system( buffer );

    char pitFileFolder[256];
    sprintf( pitFileFolder, Q_PIT_FILE_FOLDER_SHELL_COMMAND_TEMPLATE, "", userSysName );
    strcat( pitFileFolder, "/" );

    if ( marker != NULL )
    {
        // printf("\nBase file name is \"%s\"\n", ( &marker[1] ));
        sprintf( buffer, "cp \"%s\" \"%s%s\" && chmod 644 \"%s%s\"",
                 argv[1], pitFileFolder, &marker[1], pitFileFolder, &marker[1] );
#ifdef DEBUG
        syslog( LOG_NOTICE, "Launcher command: '%s'", buffer );
#endif

        system( buffer );
        sprintf( buffer, "pntsc -f \"%s%s\"", pitFileFolder, &marker[1] );
    }
    else
    {
        // printf("\nBase file name is \"%s\"\n", ( argv[1] ));
        sprintf( buffer, "cp \"%s\" \"%s%s\" && chmod 644 \"%s%s\"",
                 argv[1], pitFileFolder, argv[1], pitFileFolder, argv[1] );
        system( buffer );

#ifdef DEBUG
        syslog( LOG_NOTICE, "Launcher command: '%s'", buffer );
#endif

        sprintf( buffer, "pntsc -f \"%s%s\"", pitFileFolder, argv[1] );
    }

#ifdef DEBUG
    syslog( LOG_NOTICE, "Launcher command2: '%s'", buffer );
#endif

    system( buffer );

    exit( 0 );
}
