#include <X11/Xlib.h>
#include "appportal_updater.h"
#include <unistd.h>


#pragma pack( push, 1 )

#ifdef __cplusplus
extern "C"
{
#endif

#define F_ENTER(x)    syslog( LOG_INFO, "%s: enter", __FUNCTION__ )
#define F_LEAVE(x)    syslog( LOG_INFO, "%s: leave", __FUNCTION__ )

#define THUMBNAIL_WIDTH       (181)
#define THUMBNAIL_HEIGHT      (181)

// For Convenience
#define THUMBNAIL_UPDATE_HDR_SIZE      ( sizeof ( struct pixel_update_header ))

//////////////////////////////////////////////////////
//
// Networking declarations
//
//////////////////////////////////////////////////////
extern void * memcpy( void *, const void *, size_t );
extern char g_sessionname[];
extern int g_seamless_rdp;
extern Display * g_display;
extern Window g_wnd;

// This call is supposed to perform the image manipulation and send
// the resulting image to the correct App Porta
//
int au_dump_screenshot( XImage * image )
{
    FILE * ofile;
    float x_scale, y_scale;

    // Payload info
    int payload_size;
    int thumb_x;
    int thumb_y;
    int pixel_count;
    int header_size;
    long * pixel_buffer;
    char * payload;
    char * marker;

    struct pixel_update_header * hdr;

    char buffer2[] = "localhost";

//    F_ENTER();

    x_scale = (float) (image->width / THUMBNAIL_WIDTH) + 0.499;
    y_scale = (float) (image->height / THUMBNAIL_HEIGHT) + 0.499;

    // Attempt to keep scaling maintail the same aspect ratio.
    if ( x_scale > y_scale ) y_scale = x_scale;
    if ( y_scale > x_scale ) x_scale = y_scale;

    // Calculate the resulting thumbnail height and width
    thumb_x = (int) ((image->width)/x_scale) + 1;
    thumb_y = (int) ((image->height)/y_scale) + 1;
    header_size = (sizeof ( struct pixel_update_header ));

    payload_size = header_size + ((thumb_x * thumb_y)* sizeof ( long )) ;
    payload      = (char *) malloc( payload_size );
    hdr          = (struct pixel_update_header *) payload;
    marker       = payload;

    // Advance the marker past the header and then fill in the pixel buffer
    // syslog( LOG_INFO, "Advancing pointer by %i bytes for header", (sizeof ( struct pixel_update_header )));
    marker += header_size;
    pixel_buffer = (long * )marker;

    // syslog (LOG_INFO, "AppPortal Updater:  Scaling factors are x scale = %f, y scale = %f.\n", x_scale, y_scale );

    // Set up the thumnail image

    // DEBUG dump the image to a ppm file.
    pixel_count = 0;
    //ofile = fopen( "/tmp/pix.ppm", "w+" );
    //fprintf(ofile, "P3 %i %i 255\n", thumb_x, thumb_y);

    for (float yp=0; yp<(image->height)  ; yp+= y_scale)
        for (float xp=0; xp<(image->width) ; xp+= x_scale)
    {
        {
            long pix = XGetPixel(image, (int)xp, (int)yp);

            // DEBUG: dump this to the temp image file
            //fprintf(ofile, "%ld %ld %ld\n", pix>>16, pix>>8&255, pix&255);

            // Copy this pixel to the pixel buffer
            pixel_buffer[pixel_count] = pix;
            pixel_count ++;
        }
    }


    // DEBUG close the tmp image file
    //fclose ( ofile );

    // Now that we have all the image pixels, send the data packet across to App Portal
    // syslog( LOG_INFO, "I wrote %i pixels to the buffer", pixel_count);

    // Build the message payload header
    hdr->width     = thumb_x;
    hdr->height    = thumb_y;
    hdr->window_id = g_wnd;
    hdr->seamless  = g_seamless_rdp;
    strncpy( hdr->session_name, g_sessionname , 64);

    if ( hdr->width == 0 && hdr->height == 0 )
    {
        syslog ( LOG_WARNING, "au_dump_screenshot() is sending a close session request for %s", g_sessionname );
        hdr->action = AU_ACTION_REMOVE_SESSION;
    }
    else
    {
        hdr->action = AU_ACTION_UPDATE_THUMBNAIL;
    }


    /*
    syslog( LOG_INFO, "Checking update header, x = %i, y = %i, Session = %s",
            hdr->width,
            hdr->height,
            hdr->session_name);
    */

    au_network_send_update_message( payload, payload_size, buffer2 );

    // Sessage sent, free the memory
    free( payload );

    // syslog( LOG_INFO, "Thumbnail Update sent for '%s', for windowID '0x%08X'", g_sessionname, g_wnd);

//    F_LEAVE();

    return 0;
}

int au_end_session( char * message )
{
    struct pixel_update_header * hdr;

    hdr = ( struct pixel_update_header *) malloc (sizeof( struct pixel_update_header ));

    hdr->action = AU_ACTION_REMOVE_SESSION;
    hdr->width  = 0;
    hdr->height = 0;
    hdr->seamless = 0;
    strncpy( hdr->session_name, g_sessionname , 64);

    // Send an update with 0x0 dimensions.  This will signal AppPortal to remove the item

    au_network_send_update_message( (char * )hdr, THUMBNAIL_UPDATE_HDR_SIZE, "localhost" );

    syslog( LOG_INFO, "Received end session request with message: %s", message );
    syslog( LOG_INFO, "Sending remove thumbnail request for session '%s'", g_sessionname );

    sleep( 1 );

    free( hdr );

    return 0;
}



#ifdef __cplusplus
}
#endif


#pragma pack(pop)
