#ifndef AUDIO_INPUT_CHANNEL_H
#define AUDIO_INPUT_CHANNEL_H

#include "dynamicChannel.h"
#include "qaudio_driver.h"
#include "qrecorder.h"
#include <pthread.h>

const int audioBufferSize = 1024;


class AudioInputChannel: public DynamicChannel
{
public:
    AudioInputChannel();
    ~AudioInputChannel();
    void processMessage(STREAM s, uint32 totalLength);
    void onClose();

private:
    void processVersionPdu(STREAM s);
    void processFormatsPdu(STREAM s);
    void processOpenPdu(STREAM s);

    void sendVersionPdu();
    void sendFormatsPdu(int pcmIndex);
    void sendOpenReplyPdu(uint32 status);
    void sendIncomingDataPdu();
    void sendDataPdu(uint8* data, uint32 size);

    void createRecorder(WAVEFORMATEX* pFormat);
    void destroyRecorder();
    static void* audioProcessing(void* pThis);

private:

    enum audioInputChannelHeader { MSG_SNDIN_VERSION = 1,
                                   MSG_SNDIN_FORMATS,
                                   MSG_SNDIN_OPEN,
                                   MSG_SNDIN_OPEN_REPLY,
                                   MSG_SNDIN_DATA_INCOMING,
                                   MSG_SNDIN_DATA,
                                   MSG_SNDIN_FORMATCHANGE };

    static const uint32 maxNumberOfAudioFormats = 100;
    WAVEFORMATEX audioFormats[maxNumberOfAudioFormats];

    audio_recorder* m_pRecorder;
    pthread_t m_audioProcessingThread;
    bool m_needStop;

    uint8 m_buffer[audioBufferSize];
};

#endif
