#include "pnrdp.h"

extern int g_pnrdp_proto;
extern int g_pnrdp_is_enabled;

static VCHANNEL *pnrdp_channel;

extern int pnrdp_init_jpeg_library();
extern int pnrdp_free_jpeg_library();

// send data to the virtual channel
static void
pnrdp_send(const char *output)
{
	// use assert - as it's me who call this function, so I control the parameter
	assert(output != NULL);

	STREAM s;
	size_t len;

	len = strlen(output);
	s = channel_init(pnrdp_channel, len);
	out_uint8p(s, output, len) s_mark_end(s);

//	printf("pnrdp send:\n");
//	hexdump(s->channel_hdr + 8, s->end - s->channel_hdr - 8);

	channel_send(s, pnrdp_channel);
}

static void
pnrdp_process_request(char * buf)
{
	// use assert - as it's me who call this function, so I control the parameter
	assert(buf != NULL);

    int server_proto = 0;

    if (strncmp(buf, PNRDP_NEG_REQ_STRING, strlen(PNRDP_NEG_REQ_STRING)) == 0)
    {
        char resp[PNRDP_NEG_MESSAGE_LEN];
        int len;
        server_proto = atol(buf + strlen(PNRDP_NEG_REQ_STRING));
        if (server_proto == 0)
        {
            fprintf (stderr, "pnrdp: error: wrong protocol number.");

            // we're unable to tell server about the error, so just do nothing
            return;
        }
        g_pnrdp_proto = MIN(server_proto, PNRDP_PROTOCOL);
        len = snprintf(resp, PNRDP_NEG_MESSAGE_LEN, "%s%d", PNRDP_NEG_RSP_STRING, g_pnrdp_proto);
        resp[len] = '\0';
        pnrdp_send(resp);
    }
    else if (strncmp(buf, PNRDP_NEG_CNF_STRING, strlen(PNRDP_NEG_CNF_STRING)) == 0)
    {
        server_proto = atol(buf + strlen(PNRDP_NEG_CNF_STRING));
        if (g_pnrdp_proto == server_proto && g_pnrdp_proto != 0)
        {
            // Negotiation succeded
            g_pnrdp_is_enabled = 1;

//            printf("pnrdp: negotiation succeeded\n");
        }
    }
    else
    {
            fprintf (stderr, "pnrdp: error: unknown request.");
            return;
    }
}

// our virtual channel callback
static void
pnrdp_process(STREAM s, char *name)
{
	unsigned int pkglen;
	char *buf;

	pkglen = s->end - s->p;
	buf = xmalloc(pkglen + 1);
	STRNCPY(buf, (char *) s->p, pkglen + 1);

//	printf("pnrdp recv:\n");
//	hexdump(s->p, pkglen);

    pnrdp_process_request(buf);

	xfree(buf);
}


// These functions the initialization/deinitialization points of PNRDP module;
// rdesktop code calls them in its main procedure, during its init/deinit

RD_BOOL
pnrdp_init(void)
{
	pnrdp_channel =
		channel_register("PNRDPP", CHANNEL_OPTION_INITIALIZED | CHANNEL_OPTION_ENCRYPT_RDP,
				 pnrdp_process);

	if (pnrdp_channel == NULL ||
        pnrdp_protocol_data_init() < 0 ||
        pnrdp_init_jpeg_library() < 0)
	{
        return 0;
	}

    return 1;
}

void
pnrdp_deinit(void)
{
	pnrdp_free_jpeg_library();
    pnrdp_protocol_data_deinit();
}
