#ifndef PNSSLCLI_H
#define PNSSLCLI_H

#include <boost/aligned_storage.hpp>
#include <boost/array.hpp>
#include <boost/enable_shared_from_this.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/asio.hpp>
#include <boost/asio/ssl.hpp>

#include <string.h>

#include "handler_alloc.h"

using boost::asio::ip::tcp;

typedef boost::asio::ssl::stream < boost::asio::ip::tcp::socket > ssl_socket;

// ------------------------------------------------------------------------------
// session: representes user session
// ------------------------------------------------------------------------------
class session: public boost::enable_shared_from_this < session >
{
public:

    explicit session (boost::asio::io_service& io_service, boost::asio::ssl::context& context);

    // gets private member socket_
    tcp::socket& socket ();

    // gets private member ssl_socket_
    ssl_socket& sslsocket ();

    // starts proxy cycle
    void start (const std::string& rdphost, const std::string& username, const std::string& ticket);

private:    
    // timer handler
    void on_timer (const boost::system::error_code& e);

    // asyncronous operations handlers
    void handle_read (const boost::system::error_code & error, size_t bytes_transferred);

    void handle_server_read (const boost::system::error_code & error, size_t bytes_transferred);

    void handle_write (const boost::system::error_code & error);

    void handle_server_write (const boost::system::error_code & error);
    
    // The socket used to communicate with the client.
    tcp::socket socket_;

    // The socket used to communicate with the remote server.
    ssl_socket  ssl_socket_;

    // Buffer used to store client data.
    boost::array < char, DATA_BUFFER_SIZE > data_in_;

    // Buffer used to store remote server data.
    boost::array < char, DATA_BUFFER_SIZE > data_out_;

    // The allocators to use for handler-based custom memory allocation.
    handler_allocator allocator_in_;
    handler_allocator allocator_out_;

    // The timer used to prevent of hanging server write operation 
    boost::asio::deadline_timer timer_;
};


typedef boost::shared_ptr < session > session_ptr;

class pnssl_server
{
public:
    pnssl_server(boost::asio::io_service& io_service,
                 const tcp::endpoint& endpoint,
                 tcp::resolver::iterator & iterator,
                 const char * remote_addr,
                 const char * username,
                 const char * ticket);
    
    void handle_accept(session_ptr sess,
                       const boost::system::error_code& error);
    
private:
    boost::asio::io_service& io_service_;
    tcp::acceptor acceptor_;
    tcp::resolver::iterator endpoint_iterator_;
    
    std::string remote_addr_;
    std::string username_;
    std::string ticket_;
};

typedef boost::shared_ptr<pnssl_server> server_ptr;

#endif
