#include "rdesktop.h"
#include "qaudio_driver.h"
#include "qaudio_alsa_drv.h"
#include "qaudio_oss_drv.h"
#include "qaudio_sun_drv.h"
#include "qaudio_sgi_drv.h"
#include "qaudio_libao_drv.h"

qaudio_waveout_driver waveout_driver;
qaudio_wavein_driver wavein_driver;


int qaudio_driver_initialize ( int _driver, char * _options )
{
	memset( &waveout_driver, 0 , sizeof ( qaudio_waveout_driver ));
    switch ( _driver )
    {
#ifdef QAUDIO_ALSA
        case QDRIVER_ALSA :
            {
                alsa_driver_init ( _options ); 
                
                waveout_driver.driver = _driver;
                waveout_driver.waveOutOpen  = &alsa_waveout_open;
                waveout_driver.waveOutClose = &alsa_waveout_close;
                waveout_driver.waveOutReset = &alsa_waveout_reset;
                waveout_driver.waveOutWrite = &alsa_waveout_write;

                waveout_driver.waveInOpen = &alsa_wavein_open;
                waveout_driver.waveInClose = &alsa_wavein_close;
                waveout_driver.waveInStart = &alsa_wavein_start;
                waveout_driver.waveInStop = &alsa_wavein_stop;
                waveout_driver.waveInRecord = &alsa_wavein_record;
            }break;
#endif
#ifdef QAUDIO_OSS
        case QDRIVER_OSS:
            {
                oss_driver_init ( _options );
                
                waveout_driver.driver = _driver;
                waveout_driver.waveOutOpen  = &oss_waveout_open;
                waveout_driver.waveOutClose = &oss_waveout_close;
                waveout_driver.waveOutReset = &oss_waveout_reset;
                waveout_driver.waveOutWrite = &oss_waveout_write;

                waveout_driver.waveInOpen = &oss_wavein_open;
		waveout_driver.waveInClose = &oss_wavein_close;
		waveout_driver.waveInStart = &oss_wavein_start;
		waveout_driver.waveInStop = &oss_wavein_stop;
		waveout_driver.waveInRecord = &oss_wavein_record;
            }break;
#endif
#ifdef QAUDIO_LIBAO
        case QDRIVER_LIBAO:
            {
                libao_driver_init ( _options );            
                
                waveout_driver.driver = _driver;
                waveout_driver.waveOutOpen  = &libao_waveout_open;
                waveout_driver.waveOutClose = &libao_waveout_close;
                waveout_driver.waveOutReset = &libao_waveout_reset;
                waveout_driver.waveOutWrite = &libao_waveout_write;            
            }break;
#endif
        /*case QDRIVER_SGI:
            {
                sgi_driver_init ( _options );            
                
                waveout_driver.driver = _driver;
                waveout_driver.waveOutOpen  = &sgi_waveout_open;
                waveout_driver.waveOutClose = &sgi_waveout_close;
                waveout_driver.waveOutReset = &sgi_waveout_reset;
                waveout_driver.waveOutWrite = &sgi_waveout_write;
            }break;
        case QDRIVER_SUN:
            {
                sun_driver_init ( _options );            
                waveout_driver.driver = _driver;
                waveout_driver.waveOutOpen  = &sun_waveout_open;
                waveout_driver.waveOutClose = &sun_waveout_close;
                waveout_driver.waveOutReset = &sun_waveout_reset;
                waveout_driver.waveOutWrite = &sun_waveout_write;            
            }break;*/
       default:
            return -1;
    }
    wavein_driver = waveout_driver;
    return 0;
}
int qaudio_driver_uninitialize ()
{
    switch ( waveout_driver.driver )
    {
#ifdef QAUDIO_ALSA
        case QDRIVER_ALSA  : alsa_driver_free (); break;
#endif
#ifdef QAUDIO_OSS
        case QDRIVER_OSS   : oss_driver_free(); break;
#endif
#ifdef QAUDIO_LIBAO
        case QDRIVER_LIBAO : libao_driver_free(); break;
#endif
      /*  case QDRIVER_SGI   : sgi_driver_free(); break;
        case QDRIVER_SUN   : sun_driver_free(); break;*/
        default : return -1;
    }
    return 0;
}

