/* 

*/

#include "rdesktop.h"
#include <stdarg.h>
#include <assert.h>

#ifdef WITH_DEBUG_SEAMLESS
#define DEBUG_SEAMLESS(args) printf args;
#else
#define DEBUG_SEAMLESS(args)
#endif

static VCHANNEL *seamless_channel;
static unsigned int seamless_serial;
static char icon_buf[1024];

extern unsigned int pnts_send_message(const char *message);
extern RD_BOOL sw_window_exists(unsigned long id);


extern struct _seamless_window seamless_window;

unsigned long  gq_is_AS = 0;

extern RD_BOOL g_seamless_rdp;
/* Moving in seamless app mode */
extern unsigned long g_moving_wnd_seamless_start_move;
extern unsigned long g_moving_wnd_seamless_start_size;
extern unsigned long g_moving_wnd_seamless;
extern int g_move_x_offset_seamless;
extern int g_move_y_offset_seamless;

int qseamless_process_CR(unsigned long id, unsigned long parent, unsigned long style, unsigned long exStyle, int x, int y, int width, int height, char *title);
void qseamless_process_DE(unsigned long id);
int qseamless_process_WP(unsigned long id, int x, int y, int width, int height, unsigned long flags, unsigned long state);
void qseamless_process_AC(unsigned long id, int state);
void qseamless_process_RG(unsigned long id);
void qseamless_process_SW(unsigned long id, int state);
void qseamless_process_ST(unsigned long id, char *title);
void qseamless_process_SY(unsigned long id, unsigned long param);
void qseamless_process_SI(unsigned long id, unsigned long type, char *data);

#define PN_PROT_KEY_LEN 2

static char *
seamless_get_token(char **s)
{
	char *comma, *head;
	head = *s;

	if (!head)
		return NULL;

	comma = strchr(head, ',');
	if (comma)
	{
		*comma = '\0';
		*s = comma + 1;
	}
	else
	{
		*s = NULL;
	}

	return head;
}


RD_BOOL qseamless_process_line(const char *line)
{
	char *p, *l;
	char *tok1, *tok2, *tok3, *tok4, *tok5, *tok6, *tok7, *tok8, *tok9;
	unsigned long id, flags;
	char *endptr;

	l = xstrdup(line);
	p = l;

	DEBUG_SEAMLESS(("qseamlessrdp got:%s\n", p));

    p += PN_PROT_KEY_LEN;
	tok1 = seamless_get_token(&p);
	tok2 = seamless_get_token(&p);
	tok3 = seamless_get_token(&p);
	tok4 = seamless_get_token(&p);
	tok5 = seamless_get_token(&p);
	tok6 = seamless_get_token(&p);
	tok7 = seamless_get_token(&p);
	tok8 = seamless_get_token(&p);
	tok9 = seamless_get_token(&p);

	if (!strncmp (line, "CR", PN_PROT_KEY_LEN))
	{
        //  Format of message:
        //  CR20024,0,94c000cc,10101,303,142,417,270,Log On to Windows

        unsigned long group, parent, style, exStyle;
		int x, y, width, height;
        char *title;

		if (!tok1)
			return 1;

		id = strtoul(tok1, &endptr, 16);
		if (*endptr)
			return False;

		parent = strtoul(tok2, &endptr, 16);
		if (*endptr)
			return False;

		style = strtoul(tok3, &endptr, 16);
		if (*endptr)
			return False;

		exStyle = strtoul(tok4, &endptr, 16);
		if (*endptr)
			return False;

		x = strtol(tok5, &endptr, 0);
		if (*endptr)
			return False;

		y = strtol(tok6, &endptr, 0);
		if (*endptr)
			return False;

		width = strtol(tok7, &endptr, 0);
		if (*endptr)
			return False;

		height = strtol(tok8, &endptr, 0);
		if (*endptr)
			return False;

        title = tok9;
	
        qseamless_process_CR (id, parent, style, exStyle, x, y, width, height, title);
	}
    else if (!strncmp (line, "DE", PN_PROT_KEY_LEN))
	{
        //  Format of message:
        //  DE20024

		if (!tok1)
			return 1;

		id = strtoul(tok1, &endptr, 16);
		if (*endptr)
			return False;

        qseamless_process_DE (id);
	}
    else if (!strncmp (line, "WP", PN_PROT_KEY_LEN))  //  Window Position
	{
        //  Format of message:
        //  WP20024,0,647,195,417,270,14,1
		//  "WP%x,%x,%d,%d,%d,%d,%x,%u",
		//  &hWnd,
		//  &hWndInsertAfter,
		//  &x,
		//  &y,
		//  &cx,
		//  &cy,
		//  &uiFlags,
		//  &dwState);

		int x, y, width, height;
		unsigned long state;

		if (!tok1)
			return False;

		if (!tok8)
			return False;

		id = strtoul(tok1, &endptr, 16);
		if (*endptr)
			return False;

        if (!sw_window_exists(id))
		    return False;

		x = strtol(tok3, &endptr, 0);
		if (*endptr)
			return False;

		y = strtol(tok4, &endptr, 0);
		if (*endptr)
			return False;

		width = strtol(tok5, &endptr, 0);
		if (*endptr)
			return False;

		height = strtol(tok6, &endptr, 0);
		if (*endptr)
			return False;

		if ((width <= 0) || (height <= 0))
			return False;

        flags = strtol(tok7, &endptr, 0);
		if (*endptr)
			return False;

		state = strtoul(tok8, &endptr, 0);
		if (*endptr)
			return False;
	
        qseamless_process_WP (id, x, y, width, height, flags, state);
	}
	else if (!strncmp (line, "AC", PN_PROT_KEY_LEN))  // Activate Window
	{
        int state;

		if (!tok2)
			return False;

		id = strtoul(tok1, &endptr, 16);
		if (*endptr)
			return False;

		state = strtol(tok2, &endptr, 10);
		if (*endptr)
			return False;

        qseamless_process_AC(id, state);
	}
	else if (!strncmp (line, "RG", PN_PROT_KEY_LEN))  // Activate Window
	{
		if (!tok1)
			return False;

		id = strtoul(tok1, &endptr, 16);
		if (*endptr)
			return False;

        qseamless_process_RG(id);
	}
	else if (!strncmp (line, "SW", PN_PROT_KEY_LEN))  // Show Window
	{
        int state;

		if (!tok1)
			return False;

		id = strtoul(tok1, &endptr, 16);
		if (*endptr)
			return False;

        if (!sw_window_exists(id))
		    return False;

		state = strtol(tok2, &endptr, 10);
		if (*endptr)
			return False;

        qseamless_process_SW(id, state);
	}
	else if (!strncmp (line, "ST", PN_PROT_KEY_LEN))  // Process the SETTEXT (ST) message.
	{
		if (!tok1)
			return False;

		id = strtoul(tok1, &endptr, 16);
		if (*endptr)
			return False;

        if (!sw_window_exists(id))
	    	return False;

        qseamless_process_ST(id, tok2);
	}
	else if (!strncmp (line, "SY", PN_PROT_KEY_LEN)) // Moves or Sizes the window.
	{
        //  Format:
        //  SY29e004e,f012,105050e
		unsigned long param;

		if (!tok1)
			return False;

		id = strtoul(tok1, &endptr, 16);
		if (*endptr)
			return False;

        if (!sw_window_exists(id))
	    	return False;

		param = strtoul(tok2, &endptr, 16);
		if (*endptr)
			return False;

        qseamless_process_SY(id, param);
	}
	else if (!strncmp (line, "SI", PN_PROT_KEY_LEN))
	{
	    unsigned long type;

		if (!tok1)
			return False;

		id = strtoul(tok1, &endptr, 16);
		if (*endptr)
			return False;

    	if (!sw_window_exists(id))
	    	return False;

		type = strtoul(tok2, &endptr, 0);  //  iconType: small/big
		if (*endptr)
			return False;

        qseamless_process_SI(id, type, tok3);
	}
    else if (!strncmp (line, "AS", PN_PROT_KEY_LEN)) // next CR(CreateWindow) is our main window
    {
        gq_is_AS += 1;
    }
    //else
	//	warning("QSeamlessRDP: Non-supported request: '%s'\n", line);

	xfree(l);
	return True;
}

