// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#ifndef PNSERVER_H
#define PNSERVER_H

#include <boost/asio.hpp>

class pnsession;

//---------------------------------------------------------------------------
class pnserver : boost::noncopyable
{
  public:
    pnserver(boost::asio::io_service& io_service, unsigned short port);
    ~pnserver();

    void handle_accept(pnsession *session, const boost::system::error_code& error);

    void set_list (const std::string& broker, const std::string& list, int type = 0);
    std::string get_list (const std::string& broker, int type = 0);

    void register_session (const std::string& session_name, pnsession * session);
    void remove_session (pnsession *session);
    pnsession * find_session (const std::string& session_name);
    std::string find_session_name ( pnsession * );

    //Session instance counters
    void inc_session_count( void );
    void dec_session_count( void );

    // work with proxy list
    void use_proxy(char *addr, char *proxy);


  private:
    boost::asio::io_service& io_service_;
    boost::asio::ip::tcp::acceptor acceptor_;
    std::map < std::string, pnsession * > sessions_;
    std::map < std::string, pnsession * > gui_sessions_;
    std::map < std::string, std::string > app_lists_[3];

    int session_count;
    pnsession * master_session;
};


#endif
