// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#ifndef PNSESSION_H
#define PNSESSION_H

#include <boost/asio.hpp>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/select.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <net/if.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <ifaddrs.h>
#include <sys/inotify.h>
#include "../../common/simplexml.h"

#include <string>

#define RDP5_DISABLE_NOTHING	0x80000000
#define RDP5_NO_WALLPAPER	0x01
#define RDP5_NO_FULLWINDOWDRAG	0x02
#define RDP5_NO_MENUANIMATIONS	0x04
#define RDP5_NO_THEMING		0x08

enum
{
    BUFFER_8    = 8, 
    BUFFER_32   = 32, 
    BUFFER_64   = 64,
    BUFFER_128  = 128,
    BUFFER_256  = 256,
    BUFFER_512  = 512,
    BUFFER_1024 = 1024,
    BUFFER_2048 = 2048,
    BUFFER_4096 = 4096,
    BUFFER_131072 = 131072
};

class pnserver;

class pnsession : boost::noncopyable
{
  public:
    pnsession(boost::asio::io_service& io_service, pnserver * server);
    ~pnsession();
    boost::asio::ip::tcp::socket& socket();
    void start ();
    char * data();
    pnserver * server();


  private:    // functions
    void handle_write(const boost::system::error_code& error);
    void handle_read(const boost::system::error_code& error, size_t bytes_transferred);

    std::string get_list (const std::string& broker, int type = 0);
    void set_list (const std::string& broker, const std::string& list, int type = 0);

    pnsession * find_session (const std::string& session_name);
    void send (const char * message);
    void use_proxy(char *addr, char *proxy);

    static void *proceed(void *param);  // request proceeding thread function
    void rgs_response(SimpleString ssRes, SimpleString ssPitFile, char* pResponse);

    struct utils
    {
        static int get_free_local_ip(unsigned int *ip);
        static int find_option(char *haystack, const char *needle, char *buffer);
        static bool get_one_broker(char* broker_list, char* onebroker);
        static void wrap_brokers(char *broker_list, const char *protocol, const char *port);
        static void get_printer_list (std::string & printerList);
        static std::string get_com_port_prefix();
        static std::vector<std::string> get_com_port_list();
        static std::vector<char *> get_com_port_names(const char* findDir, const char* prefix);
    };

  private:    // data
    boost::asio::ip::tcp::socket socket_;
    pnserver * server_;
    enum { max_length = BUFFER_4096 };
    char data_[max_length];
};



#endif
