// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include "tmp.h"
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>

_tmpfile::_tmpfile(){
    Directory = new char[strlen("/tmp")+1];
    strcpy(Directory, "/tmp");
    forrand = time(NULL);
}

_tmpfile::_tmpfile(const char* dir){
    Directory = new char[strlen(dir) + 1];
    strcpy(Directory, dir);
    forrand = time(NULL);
}

_tmpfile::~_tmpfile(){
    delete(Directory);
}

char* _tmpfile::Cat(const char* NameApp, const char* FileName, const char* Prefix){
    char *result = new char[strlen(Directory) + strlen(NameApp) + strlen(Prefix) + strlen(FileName) + 3];
    strcpy(result, Directory);
    strcat(result, "/");
    strcat(result, NameApp);
    strcat(result, "/");
    strcat(result, Prefix);
    strcat(result, FileName);
    return result;
}

char* _tmpfile::CreateFile(const char* NameApp, const char* Prefix){
    char* fulldir = new char[strlen(Directory) + strlen(NameApp) + 2];
    strcpy(fulldir, Directory);
    strcat(fulldir, "/");
    strcat(fulldir, NameApp);
    mkdir(fulldir, S_IRWXU);
    char *tmp, *result;
    tmp = RandomName();
    result = Cat(NameApp, tmp, Prefix);
    while(!CheckFile(result)){
	delete(tmp);
	delete(result);
	tmp = RandomName();
	result = Cat(NameApp, tmp, Prefix);
    }
    delete(tmp);
    return result;
}

char* _tmpfile::RandomName(){
    forrand = rand_r(&forrand);
    char *tmp = new char[50];
    sprintf(tmp, "%u", forrand);
    return tmp;
}

bool _tmpfile::CheckFile(const char* FileName){
    FILE *file;
    file = fopen(FileName, "r");
    if (file){
	fclose(file);
	return false;
    }else{
	return true;
    }
}

bool _tmpfile::CleanDir(const char* NameApp){
    char* FullDir = new char[strlen(Directory) + strlen(NameApp) + 2];
    strcpy(FullDir, Directory);
    strcat(FullDir, "/");
    strcat(FullDir, NameApp);
    DIR *dir;
    struct dirent *infoDir;
    char FileName [1024];
    dir = opendir(FullDir);
    if (dir == NULL)
	return false;
    //infoDir = readdir(dir);
    while ((infoDir = readdir(dir)) /*!= NULL*/){
	if (infoDir == NULL)
	    break;
	if (!strcmp (infoDir->d_name, ".")|| !strcmp(infoDir->d_name, ".."))
	    continue;
	strcpy(FileName, FullDir);
	strcat(FileName, "/");
	strcat(FileName, infoDir->d_name);
	unlink(FileName);
    }
    delete(FullDir);
    return true;
}

std::vector<char *> splitString(char * psource)
{
	std::vector<char *> result;
	char * pch;
	pch = strtok (psource," ,");
	while (pch != NULL)
	{
		result.push_back(pch);
		pch = strtok (NULL, " ,");
	}
	return result;
}
